package us.codecraft.webmagic.selector;

import java.util.List;

import org.htmlcleaner.HtmlCleaner;
import org.htmlcleaner.TagNode;
import org.htmlcleaner.XPatherException;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.junit.Assert;
import org.junit.Ignore;
import org.junit.Test;

import org.w3c.dom.Node;
import us.codecraft.webmagic.Page;
import us.codecraft.webmagic.Spider;
import us.codecraft.webmagic.processor.PageProcessor;
import us.codecraft.xsoup.XPathEvaluator;
import us.codecraft.xsoup.Xsoup;

import javax.xml.transform.TransformerException;

/**
 * @author code4crafter@gmail.com <br> Date: 13-4-21 Time: 上午10:06
 */
public class XpathSelectorTest {

    private String html = "\n"
            + "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n"
            + "<html lang='zh-CN' xml:lang='zh-CN' xmlns='http://www.w3.org/1999/xhtml'>\n"
            + "<head>\n"
            + "  <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"/>\n"
            + "  <link rel=\"shortcut icon\" type=\"image/x-icon\" href=\"/img/favicon.ico\" />\n"
            + "  <title>再次吐槽easyui - 开源中国 OSChina.NET</title>\n"
            + "      <link rel=\"stylesheet\" href=\"/css/style2013.css?ver=20130411\" type=\"text/css\" media=\"screen\" />\n"
            + "  <link rel=\"stylesheet\" href=\"/css/channel.css?date=20130324_2\" type=\"text/css\" media=\"screen\" />\n"
            + "  <link rel=\"stylesheet\" type=\"text/css\" href=\"/js/2011/fancybox/jquery.fancybox-1.3.4.css\" media=\"screen\" />\n"
            + "  <link rel=\"alternate\" type=\"application/rss+xml\" title=\"最新代码分享列表\" href=\"http://www.oschina.net/code/rss\" />\n"
            + "  <link rel=\"alternate\" type=\"application/rss+xml\" title=\"开源中国 - 源码列表\" href=\"http://www.oschina.net/code/source_rss\" />\n"
            + "  <link rel=\"alternate\" type=\"application/rss+xml\" title=\"最新问题列表\" href=\"http://www.oschina.net/question/rss\" />\n"
            + "  <script type=\"text/javascript\" src=\"/js/2012/jquery-1.7.1.min.js\"></script>\n"
            + "  <script type=\"text/javascript\" src=\"/js/2012/jquery.form.js\"></script>\n"
            + "  <script type=\"text/javascript\" src=\"/js/2011/fancybox/jquery.fancybox-fixed.js?20130327\"></script>\n"
            + "      <link rel=\"stylesheet\" href=\"/js/poshytip/tip-yellowsimple/tip-yellowsimple.css\" type=\"text/css\" />\n"
            + "  <script type=\"text/javascript\" src=\"/js/poshytip/jquery.poshytip.min.js\"></script>\n"
            + "    <script type=\"text/javascript\">\n"
            + "  \tg_msg = {\n"
            + "};\n"
            + "\n"
            + "g_user = {\n"
            + "\tid:190591,\n"
            + "\tname:'黄亿华',\n"
            + "\tlogin:true,\n"
            + "\tbportrait:'<img src=\"http://static.oschina.net/uploads/user/95/190591_50.jpg?t=1347254905000\" align=\"absmiddle\" alt=\"黄亿华\" title=\"黄亿华\" class=\"SmallPortrait\" user=\"190591\"/>'\n"
            + "};  </script>\n"
            + "    <script type=\"text/javascript\" src=\"/js/2011/oschina.js?ver=20121007\"></script>\n"
            + "  <script type=\"text/javascript\" src=\"/js/utils.js\"></script>\n"
            + "  <script type=\"text/javascript\" src=\"/js/channel.js\"></script>\n"
            + "      <style type=\"text/css\">\n"
            + "    body,table,input,textarea,select {font-family:Verdana,sans-serif,宋体;}  \n"
            + "  </style>\n"
            + "  </head>\n"
            + "<body>\n"
            + "<div id='OSC_NavTop'>\n"
            + "\t<div class=\"wp998\">\n"
            + "        <div id=\"OSC_Channels\">\n"
            + "        \t<ul>\n"
            + "        \t<li class=\"item\"><a href=\"http://www.oschina.net/\" class='home'>首页</a></li>        \t<li class=\"item\"><a href=\"http://www.oschina.net/project\" class='project'>开源软件</a></li>\n"
            + "        \t<li class=\"item control_select\">\n"
            + "\t\t\t\t<a href=\"http://www.oschina.net/question\" class='question hl'>讨论区</a>\t\t\t\t\n"
            + "\t\t\t\t<ul class=\"cs_content\">\t\t\t\t\t\n"
            + "                \t<li><a href=\"http://www.oschina.net/question?catalog=1\"> 技术问答 &raquo; </a></li>\n"
            + "                \t<li><a href=\"http://www.oschina.net/question?catalog=2\"> 技术分享 &raquo; </a></li>\n"
            + "                \t<li><a href=\"http://www.oschina.net/question?catalog=3\"> IT大杂烩 &raquo; </a></li>\n"
            + "                \t<li><a href=\"http://www.oschina.net/question?catalog=100\"> 职业生涯 &raquo; </a></li>\n"
            + "                \t<li><a href=\"http://www.oschina.net/question?catalog=4\"> 站务/建议 &raquo; </a></li>\n"
            + "                \t<li><a href=\"http://www.oschina.net/alipay\"> 支付宝专区 &raquo; </a></li>\n"
            + "\t\t\t\t</ul>\n"
            + "\t\t\t</li>\n"
            + "        \t<li class=\"item\"><a href=\"http://www.oschina.net/code/list\" class='code'>代码分享</a></li>\n"
            + "        \t        \t<li class=\"item\"><a href=\"http://www.oschina.net/blog\" class='blog'>博客</a></li>\n"
            + "        \t<li class=\"item\"><a href=\"http://www.oschina.net/translate\" class='tran'>翻译</a></li>\n"
            + "            <li class=\"item\"><a href=\"http://www.oschina.net/news\" class='news'>资讯</a></li>\n"
            + "        \t<li class=\"item control_select\">\n"
            + "\t\t\t\t<a href=\"http://www.oschina.net/android\" class='mobile'>移动开发</a>\n"
            + "\t\t\t\t<ul class=\"cs_content cs_mobile\">\n"
            + "                \t<li class=\"android_\"><a href=\"http://www.oschina.net/android\">Android开发专区</a></li>\n"
            + "                \t<li class=\"ios_\"><a href=\"http://www.oschina.net/ios/home\">iOS开发专区</a></li>\n"
            + "                \t<li class=\"ios_\"><a href=\"http://www.oschina.net/ios/codingList\">iOS代码库</a></li>\n"
            + "                \t<li class=\"wp7_\"><a href=\"http://www.oschina.net/wp7\">WP7开发专区</a></li>\n"
            + "\t\t\t\t</ul>\n"
            + "\t\t\t</li>\n"
            + "        \t<li class=\"item t_job\"><a href=\"http://www.oschina.net/job\" class='job'>招聘</a></li>\n"
            + "        \t</ul>\n"
            + "        </div>\n"
            + "\t\t<div id=\"OSC_Userbar\">\n"
            + "                \t\t    \t\t<em>黄亿华</em>,您好 \n"
            + "\t\t\t<span class=\"control_select\">\n"
            + "\t\t\t\t<a href=\"http://my.oschina.net/flashsword\" id=\"MySpace\" title=\"我的空间\">我的空间</a>\n"
            + "\t\t\t\t<ul class=\"cs_content cs_myspace\">\n"
            + "                \t<li class='msg_'><a href='http://www.oschina.net/home/go?page=admin%2Finbox'>站内留言</a></li>\n"
            + "                \t<li class='discuss_'><a href='http://my.oschina.net/flashsword/?ft=bbs&scope=2&showme=1'>我的讨论记录</a></li>\n"
            + "                \t<li class='code_'><a href='http://www.oschina.net/code/list_by_user?id=190591'>我分享的代码</a></li>\n"
            + "                \t<li class='blog_'><a href='http://www.oschina.net/home/go?page=blog'>我的博客</a></li>\n"
            + "                \t<li class='friends_'><a href='http://www.oschina.net/home/go?page=fellow'>我关注的人</a></li>\n"
            + "                \t<li class='favorites_'><a href='http://www.oschina.net/home/go?page=favorites'>我的收藏夹</a></li>\n"
            + "                \t<li class='profile_'><a href='http://www.oschina.net/home/go?page=admin%2Fprofile'>个人资料修改</a></li>\n"
            + "                \t<li class='portrait_'><a href='http://www.oschina.net/home/go?page=admin%2Fportrait'>更改我的头像</a></li>\n"
            + "\t\t\t\t</ul>\n"
            + "\t\t\t</span>&nbsp;|&nbsp;\n"
            + "\t\t\t<a href=\"http://www.oschina.net/home/go?page=admin%2Fnew-project\">添加软件</a>&nbsp;|&nbsp;<a href=\"http://www.oschina.net/home/go?page=admin%2Fnew-release\">投递新闻</a>&nbsp;|&nbsp;<a href=\"/action/user/logout?session=6db40e6e2d1061998068&goto_page=http%3A%2F%2Fwww.oschina.net%2Fquestion%2F818848_107307\">退出</a>\n"
            + "    \t\t\t\t</div>\n"
            + "\t\t<div class='clear'></div>\n"
            + "\t</div>\n"
            + "</div>\n"
            + "<div id='OSC_Banner'><div class=\"wp998\"><a href='http://www.oschina.net/' class='Logo' title='OSChina 开源中国'>开源中国</a>\n"
            + "<h1><a href='/question'>讨论区</a></h1>\n"
            + "<dl>\n"
            + "\t<dt>当前位置：</dt>\n"
            + "\t<dd>\n"
            + "\t\t\t\t\t        \t\t<a href=\"/question\">讨论区</a>&nbsp;&raquo;\n"
            + "        \t\t<a href=\"/question?catalog=1\">技术问答</a>\t\t\t\t\t\t\t\t&raquo;&nbsp;<a href=\"/p/jquery+easyui\">EasyUI</a>\n"
            + "\t\t\t\t\t\t\t\t\t\t</dd>\n"
            + "</dl>\n"
            + "<form action='http://www.oschina.net/search' class='search'>\n"
            + "\t<input type='hidden' name='scope' value='bbs'/>\n"
            + "\t<input id='channel_q' type='text' name='q' value='' placeholder='资讯、软件、分享、代码、博客' class='TXT'/>\n"
            + "    <button type='submit' class='BTN'>搜 索</button>\n"
            + "</form>\n"
            + "<div class='clear'></div></div></div>\n"
            + "<div id=\"OSC_Screen\">\n"
            + "\t<div id=\"OSC_Content\" class='CenterDiv'>\n"
            + "<script type=\"text/javascript\" src=\"/js/scrolltopcontrol.js\"></script>\n"
            + "<script type=\"text/javascript\">\n"
            + "  \tscrolltotop.offset(100,120);\n"
            + "  \tscrolltotop.init();\n"
            + "\t$(function(){\n"
            + "\t\t$('a.ShowUserOutline img.SmallPortrait').poshytip({\n"
            + "\t\t\tclassName: 'tip-yellowsimple',\n"
            + "\t\t\talignTo: 'target',\n"
            + "\t\t\talignX: 'right',\n"
            + "\t\t\talignY: 'inner-top',\n"
            + "\t\t\toffsetX: 5,\n"
            + "\t\t\toffsetY: 0,\n"
            + "\t\t\tfade: false,\n"
            + "\t\t\tslide: false,\n"
            + "\t\t\tcontent: function(updateCallback) {\n"
            + "\t\t\t\tajax_get(\"/action/ajax/get_user_outline?id=\"+$(this).attr('user'),false,function(html){\n"
            + "\t\t\t\t\tupdateCallback(html);\n"
            + "\t\t\t\t});\n"
            + "\t\t\t\treturn \"<div style='height:100px;'>Loading...</div>\";\n"
            + "\t\t\t}\n"
            + "\t\t});\n"
            + "\t});\n"
            + "\tfunction add_to_favorite(pid,concern_it){\n"
            + "\t\t\tif(concern_it){\n"
            + "\t\t\t$(\"#p_attention_count\").load(\"/action/favorite/add?mailnotify=true&type=2&id=\"+pid, {user: '190591'});\n"
            + "\t\t\t$('#attention_it').html('<a href=\"javascript:add_to_favorite('+pid+',false)\">取消</a>');\t\n"
            + "\t\t}\n"
            + "\t\telse{\n"
            + "\t\t\t$(\"#p_attention_count\").load(\"/action/favorite/cancel?type=2&id=\"+pid, {user: '190591'});\n"
            + "\t\t\t$('#attention_it').html('<a href=\"javascript:add_to_favorite('+pid+',true)\">收藏</a>');\n"
            + "\t\t}\n"
            + "\t\t}\n"
            + "</script>\n"
            + "\n"
            + "\n"
            + "\n"
            + "<div class='Question'>\n"
            + "\t\n"
            + "\t<div class='Body'>\n"
            + "\t<div class='Title'>\n"
            + "\t\t<div class='Asker'><a href=\"http://my.oschina.net/u/818848\" class=\"ShowUserOutline\" target=\"_blank\"><img src=\"http://static.oschina.net/uploads/user/409/818848_50.jpg?t=1357353541000\" align=\"absmiddle\" alt=\"午后冬日\" title=\"午后冬日\" class=\"SmallPortrait\" user=\"818848\"/></a></div>\n"
            + "\t\t<div class='QTitle'>\n"
            + "\t\t\t<h1><a href=\"http://www.oschina.net/question/818848_107307\" hidefocus=\"true\" name='top'>再次吐槽easyui</a></h1>\n"
            + "\t\t\t<div class='stat'>\n"
            + "\t\t\t\t<a href=\"http://my.oschina.net/u/818848\" target=\"_blank\">午后冬日</a>\n"
            + "\t\t\t\t发表于 2013-4-21 02:28 13小时前,\n"
            + "\t\t\t\t<a href='#answers' class='answer_count'>3</a>回/289阅,\n"
            + "\t\t\t\t最后回答: 4小时前\t\t\t\t\t\t\t\t\t\t\t</div>\n"
            + "\t\t</div>\n"
            + "\t\t\n"
            + "\t\t<div class='clear'></div>\n"
            + "\t</div>\n"
            + "\t\t    \t    \t\t\t\t\t\n"
            + "\t\t<p style='color:#A00;font-weight:bold;margin:10px 0 0 3px;'>Java、PHP、Ruby、iOS、Python 等 JetBrains 开发工具低至 99  元（3折），<a href='http://www.oschina.net/shop/jetbrains' target='_blank'>详情&raquo;</a></p>\n"
            + "\t\t<div class='Content'>\n"
            + "\t\t\t\t\t\t<div class='detail'>刚用到easyui treegrid组件，发现这货第一次加载时候并没有传默认参数，展开某一列时候才传递id:xx的参数。这样和后台总是疙里疙瘩，像没事就拌嘴的两口子，查网上都遇到相同问题，最好解决方案就是通过 \n"
            + "<span style=\"font-family:Arial, Helvetica, 'Nimbus Sans L', sans-serif;font-size:14px;line-height:normal;background-color:#FFFFFF;\">onBeforeExpand事件来扩展，自行解决。看到官方例子中简洁的代码，感觉easyui耍流氓了，真搞不懂为何要这样实现</span><div class='clear'></div></div>\n"
            + "\t\t\t\t\t\t<div class='Tags'>\n"
            + "\t\t\t\t<strong>标签：</strong>\t\t\t\t\n"
            + "\t\t\t\t\t\t\t\t\t\t\t\t<a href='http://www.oschina.net/question/tag/jquery+easyui' class='tag project' title='jQuery的UI组件 EasyUI'>EasyUI</a> \t\t\t\t\t\t\t\t\t\t\t</div>\n"
            + "\t\t\t\t\t\t<div class='SameQuestions'>\n"
            + "\t\t\t<span id='RQuestionAction'>\n"
            + "\t\t\t\t\t\t\t\t\t<a href=\"javascript:ask_too(107307,true)\" class='rndbutton'><span>我想问同样的问题</span></a>\n"
            + "\t\t\t\t\t\t</span>\n"
            + "\t\t\t共<em id='c_asker_count'>0</em>个人想要问同样的问题\n"
            + "\t\t\t\t\t\t<a href=\"javascript:make_question_more_detail(107307)\">补充话题说明&raquo;</a>\n"
            + "\t\t\t</div>\n"
            + "\t\t\t\t\t\t<div class='EditLogs'>\n"
            + "\t<ul></ul>\n"
            + "</div>\t\t</div>\n"
            + "\t\t<style type='text/css'>\n"
            + "\t\t#favor_form{width:200px;}\n"
            + "        #favor_form p {color:#666;}\n"
            + "        #favor_form form{height:60px;width:200px;}\n"
            + "        #favor_form form ._favor_input{display:block;margin:2px 0;width:199px;}\n"
            + "        #favor_form form ._favor_button{float:left;padding:2px 5px;}\n"
            + "        .favor_ok {text-align:center;font-size:10.5pt;width:199px;height:60px;margin-top:10px;}\n"
            + "        #TagsSwitcher{cursor:pointer;float:right;margin-top:10px;}\n"
            + "        #MyTags{display:none;width:199px;}\n"
            + "        #MyTags a.tag {float:left; background-color: #E0EAF1;border-bottom: 1px solid #3E6D8E;border-right: 1px solid #7F9FB6;color: #3E6D8E;font-size: 8pt;line-height: 16px;margin: 2px 2px 2px 0;padding: 2px 4px;text-decoration: none;white-space: nowrap;}\n"
            + "\t\t.osc_promotion{ position: relative; display: inline-block; padding: 10px; margin: 10px 0; border: 1px solid #ccc;}\n"
            + "        .osc_promotion .c{ position: absolute; right: -1px; top: -1px;}\n"
            + "\t\t.ask_toolbar {float:right;list-style: none; font-size: 12px; color: #333; height: 28px;_padding-top: 5px; overflow: hidden;margin:20px 0 10px 0;}\n"
            + "        .ask_toolbar div{ float: left; margin-left: 5px; background: url(\"/img/ask-icon.gif\") no-repeat; padding: 6px 0 6px 15px; padding-left: 15px; height: 16px;}\n"
            + "        .ask_toolbar a{ height: 16px; color: #333; text-decoration: none; display:inline-block; zoom:1; vertical-align: middle; }\n"
            + "        .ask_share{width: 89px;vertical-align: bottom; line-height: 15px; _line-height: 14px;}\n"
            + "        .ask_share a{background: url(\"/img/ask-icon.gif\"); width: 16px; }\n"
            + "        a.ask_share_sina{ background: url(\"/img/ask-icon.gif\") 0 -40px no-repeat; margin-left: 5px;  }\n"
            + "        a.ask_share_tencent{background-position: 0 -70px; margin-left: 5px; }\n"
            + "        .ask_toolbar em{ height: 28px; line-height:28px; width: 14px; display: inline-block; float: left; background: url(\"/img/ask-icon.gif\") top right;}\n"
            + "        .ask_collect a,.ask_report a, .ask_vote a, .ask_collected a{padding-left: 20px; line-height: 15px; }\n"
            + "        .ask_collect a{ background: url(\"/img/ask-icon.gif\") 0 -131px no-repeat; }\n"
            + "        .ask_collected a{ background: url(\"/img/ask-icon.gif\") 0 -100px no-repeat; }\n"
            + "        div.ask_collect_count{ background: url(\"/img/ask-icon.gif\") 0 -309px no-repeat; font-weight: bold; font-size: 14px; margin-left: 0; height: 16px;line-height: 16px;}\n"
            + "        .ask_report a{ background: url(\"/img/ask-icon.gif\") 0 -160px no-repeat;}\n"
            + "        em.ask_collect_count_r{background-position: 59px -309px;}\n"
            + "        .ask_vote a{background: url(\"/img/ask-icon.gif\");}\n"
            + "        \t\ta.ask_vote_up{background-position: 3px -190px;}\n"
            + "        a.ask_vote_down{background-position: 0 -280px;}\n"
            + "        a.ask_vote_uped {background-position: 3px -190px;}\n"
            + "        \t\ta.ask_vote_downed {background-position: 0 -280px;}\n"
            + "        .ask_vote span{ display: inline-block; margin: 0 10px; font-weight: bold; font-size: 14px; vertical-align: middle; margin-bottom: 2px; line-height: 16px;}\n"
            + "\t\tspan.vote-down-count{margin:0 3px;}\n"
            + "\t\tspan.vote-up-count{margin:0 3px;}\n"
            + "\t\t</style>\n"
            + "\t\t<div class='clear'></div>\n"
            + "        \n"
            + "\t\t\t\t<div class=\"ask_toolbar\">\n"
            + "\t\t\t<div class=\"ask_share\"><b>分享到</b> <a class=\"ask_share_sina\" title=\"分享到新浪微博\"  href=\"javascript:void((function(s,d,e,r,l,p,t,z,c){var%20f='http://v.t.sina.com.cn/share/share.php?appkey=858381728',u=z||d.location,p=['&url=',e(u),'&title=',e(t||d.title),'&source=',e(r),'&sourceUrl=',e(l),'&content=',c||'gb2312','&pic=',e(p||'')].join('');function%20a(){if(!window.open([f,p].join(''),'mb',['toolbar=0,status=0,resizable=1,width=440,height=430,left=',(s.width-440)/2,',top=',(s.height-430)/2].join('')))u.href=[f,p].join('');};if(/Firefox/.test(navigator.userAgent))setTimeout(a,0);else%20a();})(screen,document,encodeURIComponent,'','','','再次吐槽easyui: 刚用到easyui treegrid组件，发现这货第一次加载时候并没有传默认参数，展开某一列时候才传递id:xx的参数。这样和后台总是疙里疙瘩，像没事就拌嘴的两口子，查网上都遇到...','','utf-8'));\"></a><a class=\"ask_share_tencent\" title=\"分享到腾讯微博\"  href=\"javascript:(function(){window.open('http://v.t.qq.com/share/share.php?url='+encodeURIComponent(document.location)+'&amp;appkey=96f54f97c4de46e393c4835a266207f4&amp;site=&amp;title='+encodeURIComponent(document.title)+encodeURIComponent(': 刚用到easyui treegrid组件，发现这货第一次加载时候并没有传默认参数，展开某一列时候才传递id:xx的参数。这样和后台总是疙里疙瘩，像没事就拌嘴的两口子，查网上都遇到...'),'', 'width=450, height=400, top=0, left=0, toolbar=no, menubar=no, scrollbars=no, location=yes, resizable=no, status=no');}())\"></a></div><em></em>\n"
            + "\t\t\t<div class=\"ask_collect\"><a title=\"收藏此话题\" id=\"favor_trigger\" href=\"javascript:;\">收藏</a></div><em></em>\n"
            + "\t\t\t<div class=\"ask_collect_count\" id=\"p_favor_count\">1</div><em class=\"ask_collect_count_r\"></em>\n"
            + "\t\t\t<div class=\"ask_report\"><a href=\"javascript:report('http://www.oschina.net/question/818848_107307',107307,2)\">举报</a></div><em></em>\n"
            + "\t\t\t<div class='ask_vote' id='Vote'>\n"
            + "\t\t\t\t\t\t\t\t        \t\t\t<a id=\"vote_down\" class=\"ask_vote_down\" href=\"javascript:;\" title=\"踩：这问题不知道在说什么，或者没什么用\">踩</a>\n"
            + "\t\t\t\t\t\t\t\t<span class='vote-down-count'>0</span>\n"
            + "\t\t\t\t|\n"
            + "\t\t\t\t\t\t\t\t        \t\t\t<a id=\"vote_up\" class=\"ask_vote_up\" href=\"javascript:;\" title=\"顶：这问题很有用或者很清晰明了\">顶</a>\n"
            + "\t\t\t\t\t\t\t\t<span class='vote-up-count'>0</span>\n"
            + "\t\t\t</div>\n"
            + "\t\t\t<em></em>\n"
            + "\t\t</div>\n"
            + "\t\t<div class='clear'></div>\n"
            + "\t\t\t\t\t\t        <div class='QuestionReplies'>\n"
            + "\t\t\t\n"
            + "        \t<h2>\t\t\t\n"
            + "\t\t\t\t<span class='sort'>\n"
            + "\t\t\t\t\t<a href=\"http://www.oschina.net/question/818848_107307#answers\" class='current'>按评价排序</a>&nbsp;|\n"
            + "\t\t\t\t\t<a href=\"?sort=time#answers\">显示最新答案</a>&nbsp;|&nbsp;<a href=\"#top\" style='padding-left:0;'>回页面顶部</a>\n"
            + "\t\t\t\t</span>\n"
            + "\t\t\t\t<a name='answers'></a>共有<em class='answer_count'>3</em>个答案 <a href=\"#answerform\" class='answer'>我要回答&raquo;</a>\n"
            + "\t\t\t</h2>\n"
            + "\t\t\t        \t<ul class='list'><li class='Answer' id='answer_467005'>\n"
            + "\t<div class='user'><a href=\"http://my.oschina.net/u/224858\" class=\"ShowUserOutline\" name=\"AnchorAnswer467005\" target=\"_blank\"><img src=\"http://static.oschina.net/uploads/user/112/224858_50.jpg\" align=\"absmiddle\" alt=\"布谷鸟\" title=\"布谷鸟\" class=\"SmallPortrait\" user=\"224858\"/></a></div>\n"
            + "\t<div class='body'>\n"
            + "\t\t<div class='time'><a href=\"http://my.oschina.net/u/224858\" target=\"_blank\">布谷鸟</a> 回答于 2013-04-21 09:28 </div>\t\t\n"
            + "    \t<div class='opts'>\n"
            + "\t\t\t    \t\t    \t\t<a href=\"javascript:report('http://www.oschina.net/question/818848_107307#AnchorAnswer467005',467005,17)\">举报</a>\n"
            + "    \t</div>\n"
            + "\t\t<div class='clear'></div>\n"
            + "\t\t<div class='detail'>对话框、日期控件和选项卡效果还不错，树状菜单没得zTree好用，建议楼主不要全部效果都依赖于此框架，有些easyui实现不好的地方可以换其它的插件实现嘛，反正我现在再也不用诸如ext和easyui之类的东西了，感觉好肥</div>\n"
            + "\t</div>\n"
            + "\t<div class='clear'></div>\n"
            + "\t<div class='replies' id='PostReplies_467005'><strong>--- 共有 1 条评论 --- </strong>\n"
            + "<ul>\n"
            + "\t\t<li id='PostReply_467044'>\n"
            + "\t\t<a href=\"http://my.oschina.net/u/818848\" class='p' name='AnchorAnswer467044'><img src=\"http://static.oschina.net/uploads/user/409/818848_50.jpg?t=1357353541000\" align=\"absmiddle\" alt=\"午后冬日\" title=\"午后冬日\" class=\"SmallPortrait\" user=\"818848\"/></a>\n"
            + "\t\t<span class='b'>\n"
            + "\t\t<span class='c'>前端水平实在有限，自己搞的总是感觉不伦不类，只能用这些框架，再集成其它插件，切换主题时风格又不一致。</span>\n"
            + "\t\t<span class='t'>(4小时前 by 午后冬日)</span>\n"
            + "\t\t<span class='opts'><a href=\"javascript:reply_to_post(467005,818848)\">回复</a></span>\n"
            + "\t\t</span>\n"
            + "\t\t<div class='clear'></div>\n"
            + "\t</li>\n"
            + "\t</ul>\n"
            + "<div class='PagerLinks'>\n"
            + "</div>\n"
            + "</div>\n"
            + "\t<div class='votes'>\t\t\t\t\t\t<a id='a_post_voteup_467005' href=\"javascript:vote_answer(467005,true,true)\" title=\"这是一个好答案，能解决问题\">有帮助</a><em id='post_voteup_467005'>(1)</em>&nbsp;|\n"
            + "\t\t<a id='a_post_votedown_467005' href=\"javascript:vote_answer(467005,false,true)\" title=\"这答案无法解决问题，或者模糊不清\">没帮助</a><em id='post_votedown_467005'>(0)</em>&nbsp;|\n"
            + "\t\t<a href=\"javascript:reply_to_post(467005, 0)\">评论</a><em>(1)</em>&nbsp;|\n"
            + "    \t<a href=\"/question/answer?question=107307&amp;answer=467005\">引用此答案</a>\t</div>\n"
            + "</li><li class='Answer' id='answer_467039'>\n"
            + "\t<div class='user'><a href=\"http://my.oschina.net/rox\" class=\"ShowUserOutline\" name=\"AnchorAnswer467039\" target=\"_blank\"><img src=\"http://static.oschina.net/uploads/user/0/180_50.jpg\" align=\"absmiddle\" alt=\"静风流云\" title=\"静风流云\" class=\"SmallPortrait\" user=\"180\"/></a></div>\n"
            + "\t<div class='body'>\n"
            + "\t\t<div class='time'><a href=\"http://my.oschina.net/rox\" target=\"_blank\">静风流云</a> 回答于 2013-04-21 11:08 </div>\t\t\n"
            + "    \t<div class='opts'>\n"
            + "\t\t\t    \t\t    \t\t<a href=\"javascript:report('http://www.oschina.net/question/818848_107307#AnchorAnswer467039',467039,17)\">举报</a>\n"
            + "    \t</div>\n"
            + "\t\t<div class='clear'></div>\n"
            + "\t\t<div class='detail'><p> 没办法，原来项目也是因为客户特殊的需求，对layout选型的时候，犹豫了好久，最终放弃了。<br /> 幸亏来了一个厉害的前端，解决问题，够用就好。 </p></div>\n"
            + "\t</div>\n"
            + "\t<div class='clear'></div>\n"
            + "\t<div class='replies' id='PostReplies_467039'><strong>--- 共有 1 条评论 --- </strong>\n"
            + "<ul>\n"
            + "\t\t<li id='PostReply_467046'>\n"
            + "\t\t<a href=\"http://my.oschina.net/u/818848\" class='p' name='AnchorAnswer467046'><img src=\"http://static.oschina.net/uploads/user/409/818848_50.jpg?t=1357353541000\" align=\"absmiddle\" alt=\"午后冬日\" title=\"午后冬日\" class=\"SmallPortrait\" user=\"818848\"/></a>\n"
            + "\t\t<span class='b'>\n"
            + "\t\t<span class='c'>我也是犹豫了好久，看过很多前端框架，总是不太满意。个人开发前台后台数据库全部要自己搞定，郁闷ing</span>\n"
            + "\t\t<span class='t'>(4小时前 by 午后冬日)</span>\n"
            + "\t\t<span class='opts'><a href=\"javascript:reply_to_post(467039,818848)\">回复</a></span>\n"
            + "\t\t</span>\n"
            + "\t\t<div class='clear'></div>\n"
            + "\t</li>\n"
            + "\t</ul>\n"
            + "<div class='PagerLinks'>\n"
            + "</div>\n"
            + "</div>\n"
            + "\t<div class='votes'>\t\t\t\t\t\t<a id='a_post_voteup_467039' href=\"javascript:vote_answer(467039,true,true)\" title=\"这是一个好答案，能解决问题\">有帮助</a><em id='post_voteup_467039'>(0)</em>&nbsp;|\n"
            + "\t\t<a id='a_post_votedown_467039' href=\"javascript:vote_answer(467039,false,true)\" title=\"这答案无法解决问题，或者模糊不清\">没帮助</a><em id='post_votedown_467039'>(0)</em>&nbsp;|\n"
            + "\t\t<a href=\"javascript:reply_to_post(467039, 0)\">评论</a><em>(1)</em>&nbsp;|\n"
            + "    \t<a href=\"/question/answer?question=107307&amp;answer=467039\">引用此答案</a>\t</div>\n"
            + "</li><li class='Answer' id='answer_467051'>\n"
            + "\t<div class='user'><a href=\"http://my.oschina.net/u/224858\" class=\"ShowUserOutline\" name=\"AnchorAnswer467051\" target=\"_blank\"><img src=\"http://static.oschina.net/uploads/user/112/224858_50.jpg\" align=\"absmiddle\" alt=\"布谷鸟\" title=\"布谷鸟\" class=\"SmallPortrait\" user=\"224858\"/></a></div>\n"
            + "\t<div class='body'>\n"
            + "\t\t<div class='time'><a href=\"http://my.oschina.net/u/224858\" target=\"_blank\">布谷鸟</a> 回答于 2013-04-21 11:29 </div>\t\t\n"
            + "    \t<div class='opts'>\n"
            + "\t\t\t    \t\t    \t\t<a href=\"javascript:report('http://www.oschina.net/question/818848_107307#AnchorAnswer467051',467051,17)\">举报</a>\n"
            + "    \t</div>\n"
            + "\t\t<div class='clear'></div>\n"
            + "\t\t<div class='detail'><div class=\"ref\"><h4>引用来自“布谷鸟”的答案</h4><div class=ref_body>对话框、日期控件和选项卡效果还不错，树状菜单没得zTree好用，建议楼主不要全部效果都依赖于此框架，有些easyui实现不好的地方可以换其它的插件实现嘛，反正我现在再也不用诸如ext和easyui之类的东西了，感觉好肥</div></div><div class=a_body>前后端你一个人搞啊？那确实很麻烦。面面俱到的话，工作量很大。但是如果需要实现的功能不是很多，而时间也不紧迫的话，事情干起来也还不错。如非必须，建议逐步弃用这些前端框架，在一些比较能够提升体验的地方选用一些适当的插件即可，如此也不再需要担心风格的问题，你看osc后台截图，界面那叫一个丑，用得方便顺手就够了</div></div>\n"
            + "\t</div>\n"
            + "\t<div class='clear'></div>\n"
            + "\t<div class='replies' id='PostReplies_467051'></div>\n"
            + "\t<div class='votes'>\t\t\t\t\t\t<a id='a_post_voteup_467051' href=\"javascript:vote_answer(467051,true,true)\" title=\"这是一个好答案，能解决问题\">有帮助</a><em id='post_voteup_467051'>(0)</em>&nbsp;|\n"
            + "\t\t<a id='a_post_votedown_467051' href=\"javascript:vote_answer(467051,false,true)\" title=\"这答案无法解决问题，或者模糊不清\">没帮助</a><em id='post_votedown_467051'>(0)</em>&nbsp;|\n"
            + "\t\t<a href=\"javascript:reply_to_post(467051, 0)\">评论</a><em>(0)</em>&nbsp;|\n"
            + "    \t<a href=\"/question/answer?question=107307&amp;answer=467051\">引用此答案</a>\t</div>\n"
            + "</li></ul>\n"
            + "\t\t\t\t        </div> \n"
            + "\t\t<div class='AnswerForm'>\n"
            + "\t\t\t<div class='user'><a href=\"http://my.oschina.net/flashsword\" name=\"answerform\"><img src=\"http://static.oschina.net/uploads/user/95/190591_50.jpg?t=1347254905000\" align=\"absmiddle\" alt=\"黄亿华\" title=\"黄亿华\" class=\"SmallPortrait\" user=\"190591\"/></a></div>\n"
            + "\t\t\t<form id='form_answer' action=\"/action/question/answer?question=107307\" method=\"post\">\n"
            + "\t\t\t\t<input type='hidden' name='user' value='190591'/>\n"
            + "\t\t\t\t<textarea id='txt_answner' name='body' style='width:560px;height:160px;'></textarea>\n"
            + "\t\t\t\t<input type='submit' value=' 我要回答 ' id=\"FormSubmitButton\" class='rndbutton'/>\n"
            + "\t\t\t\t<span id='form_msg' style='display:none;'></span>\n"
            + "\t\t\t\t<br><br>\n"
            + "\t\t\t\t<a href=\"#answers\">回答案顶部</a>&nbsp;|&nbsp;<a href=\"#top\">回页面顶部</a>\n"
            + "\t\t\t</form>\n"
            + "\t\t\t<div class='clear'></div>\n"
            + "\t\t\t<script>\t\t\t\n"
            + "            $('#form_answer').ajaxForm({\n"
            + "            \tdataType: 'json',\n"
            + "        \t\tbeforeSerialize: function($form, options) { \n"
            + "        \t\t\teditor.sync();           \n"
            + "                },\n"
            + "        \t\tbeforeSubmit: function(){\n"
            + "        \t\t\t$('#FormSubmitButton').attr('disabled','disabled');\n"
            + "        \t\t\t$('#form_msg').html(\"<span class='ajax_processing'>正在提交答案，请稍候...</span>\");\t\n"
            + "        \t\t\t$('#form_msg').show();\t\n"
            + "        \t\t},\n"
            + "                success: function(json) {\n"
            + "        \t\t\t$('#FormSubmitButton').removeAttr('disabled');\n"
            + "            \t\tif(json.msg){\n"
            + "        \t\t\t\t$('#form_msg').html(\"<span class='error_msg'>\"+json.msg+\"</span>\");\n"
            + "        \t\t\t\t$('#form_msg').show();\n"
            + "        \t\t\t}\n"
            + "            \t\telse if(json.id){\n"
            + "            \t\t\tajax_get(\"/question/show_answer?_answer_id=\"+json.id, true, function(data){\n"
            + "        \t\t\t\t\t            \t\t\t\t$('.QuestionReplies ul.list').append(data);\n"
            + "        \t\t\t\t\t        \t\t\t\t\teditor.html('');\n"
            + "        \t\t\t\t\t$('.answer_count').html(json.answer_count);\n"
            + "            \t\t\t}); \t\t\t\t\n"
            + "        \t\t\t\t$('#form_msg').hide();\n"
            + "            \t\t}\n"
            + "                }\n"
            + "            });\n"
            + "\t\t\t</script>\n"
            + "\t\t</div>\n"
            + "\t</div>\t\n"
            + "\t<script type=\"text/javascript\" src=\"/js/syntax-highlighter-2.1.382/scripts/brush.js\"></script>\n"
            + "<link type=\"text/css\" rel=\"stylesheet\" href=\"/js/syntax-highlighter-2.1.382/styles/shCore.css\"/>\n"
            + "<link type=\"text/css\" rel=\"stylesheet\" href=\"/js/syntax-highlighter-2.1.382/styles/shThemeDefault.css\"/>\n"
            + "<script type='text/javascript'><!--\n"
            + "$(document).ready(function(){\n"
            + "\tSyntaxHighlighter.config.clipboardSwf = '/js/syntax-highlighter-2.1.382/scripts/clipboard.swf';\n"
            + "\tSyntaxHighlighter.all();\n"
            + "});\n"
            + "//-->\n"
            + "</script>\n"
            + "\t<div class='QuestionRelations'>\n"
            + "\t <div style='text-align:center;margin-bottom:10px;'>\n"
            + "    \t<a href=\"http://www.oschina.net/action/visit/ad?id=1033\" target=\"_blank\" title=\"JPush——极光推送\"><img src=\"http://static.oschina.net/uploads/space/2013/0319/103739_17pH_179699.jpg\"/></a>\n"
            + "\t</div>\n"
            + "\t\t<div id='QuestionWizard'>\n"
            + "\t\t\t有什么技术问题吗？\n"
            + "\t\t\t<a href='/question/ask' class='rndbutton'><span>我要提问</span></a>\n"
            + "\t\t\t<div class='clear'></div>\n"
            + "\t\t</div>\n"
            + "\t\t\n"
            + "\t\t\t\t\t\t<div id='OtherQuestionsOfUser' class='Qlist'>\n"
            + "\t\t\t<strong><a href=\"http://my.oschina.net/u/818848/?ft=bbs&scope=2&showme=1\" class=\"more\">全部(29)...</a><em>午后冬日</em>的其他问题</strong>\n"
            + "\t\t\t<ul>\n"
            + "\t\t\t\t\t\t\t\t\t\t\t\t<li><a href=\"http://www.oschina.net/question/818848_106829\" title=\"是jsf还是ajax框架，这是个问题\">是jsf还是ajax框架，这是个问题</a><span class='date'>(4回/194阅,4天前)</span></li>\t\t\t\t\t\t<li><a href=\"http://www.oschina.net/question/818848_106805\" title=\"关于ireport的问题\">关于ireport的问题</a><span class='date'>(0回/4阅,4天前)</span></li>\t\t\t\t\t\t<li><a href=\"http://www.oschina.net/question/818848_106539\" title=\"关于JasperReports的问题\">关于JasperReports的问题</a><span class='date'>(2回/47阅,6天前)</span></li>\t\t\t\t\t\t<li><a href=\"http://www.oschina.net/question/818848_105917\" title=\"IDEA代码编辑窗口能不能上下分屏\">IDEA代码编辑窗口能不能上下分屏</a><span class='date'>(2回/53阅,10天前)</span></li>\t\t\t\t\t\t</ul>\n"
            + "\t\t</div>\n"
            + "\t\t\t\t<div style='text-align:center;margin-top:20px;'>\n"
            + "\t\t<script type=\"text/javascript\" src=\"/js/ad/question.js\"></script>\n"
            + "\t\t</div>\n"
            + "\t\t\n"
            + "\t\t<div id='Similarity' class='Qlist'>\n"
            + "\t\t\t<strong>类似的话题</strong>\n"
            + "\t\t\t<ul>\n"
            + "        \t\t\t\t<li><a href=\"http://www.oschina.net/question/267632_49688\" title=\"jQuery easyUI 分页(Pagination)用法\">jQuery easyUI 分页(Pagination)用法</a><span class='date'>(2回/1228阅,1年前)</span></li>\t\t\t\t\t\t<li><a href=\"http://www.oschina.net/question/138848_49846\" title=\"谨慎使用EasyUI\">谨慎使用EasyUI</a><span class='date'>(1回/1361阅,1年前)</span></li>\t\t\t\t\t\t<li><a href=\"http://www.oschina.net/question/205548_31992\" title=\"easyui datagird 初始化加载俩次\">easyui datagird 初始化加载俩次</a><span class='date'>(6回/690阅,1年前)</span></li>\t\t\t\t\t\t<li><a href=\"http://www.oschina.net/question/84535_32061\" title=\"打算使用easyui,求源码,求建议~\">打算使用easyui,求源码,求建议~</a><span class='date'>(17回/4105阅,1年前)</span></li>\t\t\t\t\t\t<li><a href=\"http://www.oschina.net/question/201422_32682\" title=\"jquery easyui form 有没好的设计\">jquery easyui form 有没好的设计</a><span class='date'>(2回/812阅,1年前)</span></li>\t\t\t\t\t\t<li><a href=\"http://www.oschina.net/question/183509_32885\" title=\"jeasyui 中combobox的onselect事件怎么做下拉框的级联\">jeasyui 中combobox的onselect事件怎么做下拉框的级联</a><span class='date'>(1回/741阅,1年前)</span></li>\t\t\t\t\t\t<li><a href=\"http://www.oschina.net/question/161511_36411\" title=\"easyui  treegrid行编辑 效率慢怎么解决?\">easyui  treegrid行编辑 效率慢怎么解决?</a><span class='date'>(1回/671阅,1年前)</span></li>\t\t\t\t\t\t<li><a href=\"http://www.oschina.net/question/166022_24392\" title=\"ComboGrid怎么动态绑定ds呢？\">ComboGrid怎么动态绑定ds呢？</a><span class='date'>(2回/427阅,1年前)</span></li>\t\t\t\t\t\t<li><a href=\"http://www.oschina.net/question/146658_24974\" title=\"easyui中怎么显示一个list的数据？\">easyui中怎么显示一个list的数据？</a><span class='date'>(1回/594阅,1年前)</span></li>\t\t\t\t\t\t<li><a href=\"http://www.oschina.net/question/97507_26165\" title=\"jquery easyui 組件無法顯示\">jquery easyui 組件無法顯示</a><span class='date'>(2回/671阅,1年前)</span></li>\t\t\t\t\t\t<li><a href=\"http://www.oschina.net/question/46586_16818\" title=\"关于jQuery EasyUI Form的问题\">关于jQuery EasyUI Form的问题</a><span class='date'>(2回/1103阅,2年前)</span></li>\t\t\t\t\t\t<li><a href=\"http://www.oschina.net/question/59256_17359\" title=\"jquery easyUI \">jquery easyUI </a><span class='date'>(1回/557阅,2年前)</span></li>\t\t\t\t\t\t<li><a href=\"http://www.oschina.net/question/188775_77569\" title=\"easyui中有没有点击datagrid的一个单元格 ，就使这个单元格变成可编辑的办法呢 \">easyui中有没有点击datagrid的一个单元格 ，就使这个单元格变成可编辑的办法呢 </a><span class='date'>(1回/890阅,5个月前)</span></li>\t\t\t\t\t\t<li><a href=\"http://www.oschina.net/question/256315_79207\" title=\"jQuery easyui：点击tree控件后无法获取node属性\">jQuery easyui：点击tree控件后无法获取node属性</a><span class='date'>(3回/533阅,5个月前)</span></li>\t\t\t\t\t\t<li><a href=\"http://www.oschina.net/question/580112_79227\" title=\"EasyUI 中tab标签的选项卡 置于左边是怎么实现的\">EasyUI 中tab标签的选项卡 置于左边是怎么实现的</a><span class='date'>(5回/426阅,5个月前)</span></li>\t\t\t\t\t\t<li><a href=\"http://www.oschina.net/question/868642_79423\" title=\"easyui无法跳转到 指定action   各位帮忙看看哪里有问题么？\">easyui无法跳转到 指定action   各位帮忙看看哪里有问题么？</a><span class='date'>(2回/162阅,5个月前)</span></li>\t\t\t\t\t\t</ul>\n"
            + "\t\t</div>\n"
            + "\t</div>\n"
            + "\t<div class='clear'></div>   \n"
            + "</div>\n"
            + "<script type='text/javascript' src='/js/ke/kindeditor-min.js?v=4.1.4' charset='utf-8'></script>\n"
            + "<script type='text/javascript'>\n"
            + "<!--\n"
            + "var editor;\n"
            + "KindEditor.ready(function(K) {\n"
            + "    editor = K.create('#txt_answner', {\n"
            + "\t\tthemeType : 'oschina',\n"
            + "\t\tresizeType : 1,\n"
            + "\t\turlType: 'domain',\n"
            + "\t\tshadowMode : false,\n"
            + "\t\tallowPreviewEmoticons : false,\n"
            + "\t\tallowImageUpload : true,\n"
            + "\t\tallowFlashUpload : false,\n"
            + "\t\tcssPath : '/css/ke-oschina.css',\n"
            + "\t\tuploadJson : '/action/blog/upload_img',\n"
            + "\t\tafterCreate : function(){\n"
            + "\t\t\t/*\n"
            + "\t\t\tK.ctrl(this.edit.iframe.get(0).contentWindow.document, 13, function() {\n"
            + "\t\t\t\t$(\"#txt_answner\").parent().submit();\n"
            + "\t\t\t});\n"
            + "\t\t\t*/\n"
            + "\t\t\t$(this.edit.iframe.get(0).contentWindow.document).keydown(function(e) {\n"
            + "\t\t\t\tif ((e.ctrlKey || e.metaKey) && e.which == 13 && !e.shiftKey && !e.altKey){\n"
            + "\t\t\t\t\t$(\"#txt_answner\").parent().submit();\n"
            + "\t\t\t\t}\n"
            + "\t\t\t});\n"
            + "\t\t},\n"
            + "\t\tafterChange : function() {\n"
            + "\t\t\tthis.sync();\n"
            + "\t\t},\n"
            + "\t\titems : ['bold', 'italic', 'underline', 'strikethrough', 'removeformat','|','insertorderedlist', 'insertunorderedlist', \n"
            + "\t\t\t\t 'forecolor', 'hilitecolor', 'fontname', 'fontsize',  '|', 'link', 'unlink', 'emoticons', \n"
            + "\t\t\t\t 'shcode', 'image', 'flash', 'quote', '|', 'source','about'],\n"
            + "\t\thtmlTags:\n"
            + "\t\t{\n"
            + "\t\t\tscript : ['src'],\n"
            + "            font : ['color', 'size', 'face', '.background-color'],\n"
            + "            span : [\n"
            + "                    '.color', '.background-color', '.font-size', '.font-family', '.background',\n"
            + "                    '.font-weight', '.font-style', '.text-decoration', '.vertical-align', '.line-height'\n"
            + "            ],\n"
            + "            div : [\n"
            + "                    'class', 'align', '.border', '.margin', '.padding', '.text-align', '.color',\n"
            + "                    '.background-color', '.font-size', '.font-family', '.font-weight', '.background',\n"
            + "                    '.font-style', '.text-decoration', '.vertical-align', '.margin-left'\n"
            + "            ],\n"
            + "            table: [\n"
            + "                    'border', 'cellspacing', 'cellpadding', 'width', 'height', 'align', 'bordercolor',\n"
            + "                    '.padding', '.margin', '.border', 'bgcolor', '.text-align', '.color', '.background-color',\n"
            + "                    '.font-size', '.font-family', '.font-weight', '.font-style', '.text-decoration', '.background',\n"
            + "                    '.width', '.height', '.border-collapse'\n"
            + "            ],\n"
            + "            'td,th': [\n"
            + "                    'align', 'valign', 'width', 'height', 'colspan', 'rowspan', 'bgcolor',\n"
            + "                    '.text-align', '.color', '.background-color', '.font-size', '.font-family', '.font-weight',\n"
            + "                    '.font-style', '.text-decoration', '.vertical-align', '.background', '.border'\n"
            + "            ],\n"
            + "            a : ['href', 'target', 'name'],\n"
            + "            embed : ['src', 'width', 'height', 'type', 'loop', 'autostart', 'quality', '.width', '.height', 'align', 'allowscriptaccess'],\n"
            + "            img : ['src', 'width', 'height', 'border', 'alt', 'title', 'align', '.width', '.height', '.border'],\n"
            + "            'p,ol,ul,li,blockquote,h1,h2,h3,h4,h5,h6' : [\n"
            + "                    'align', '.text-align', '.color', '.background-color', '.font-size', '.font-family', '.background',\n"
            + "                    '.font-weight', '.font-style', '.text-decoration', '.vertical-align', '.text-indent', '.margin-left'\n"
            + "            ],\n"
            + "            pre : ['class'],\n"
            + "            hr : ['class', '.page-break-after'],\n"
            + "            'br,tbody,tr,strong,b,sub,sup,em,i,u,strike,s,del' : []\n"
            + "\t\t}\n"
            + "    });\n"
            + "});\n"
            + "//-->\n"
            + "</script>\n"
            + "<!--[if lt IE 7]>\n"
            + "<script type=\"text/javascript\" src=\"/js/minmax.js\"></script>\n"
            + "<![endif]-->\n"
            + "<script type=\"text/javascript\" src=\"/action/visit/question?id=107307\"></script>\n"
            + "<script type='text/javascript'>\n"
            + "<!--\n"
            + "$(document).ready(function() {\n"
            + "\t$('.Answer .replies li').hover(\n"
            + "\t\tfunction(){$(this).addClass('hover');},\n"
            + "\t\tfunction(){$(this).removeClass('hover');}\n"
            + "\t);\t\n"
            + "\t\n"
            + "    $('.detail img').css('cursor','pointer');\n"
            + "    jQuery.each($('.detail img'),function(idx,v){\n"
            + "    \t$(v).wrap(\"<a href='\"+$(this).attr('src')+\"' target='_blank'></a>\");\n"
            + "    });\n"
            + "\t\n"
            + "\t$('#c').bind('mouseover mouseout',function(){\n"
            + "\t\t$('#c_on').toggle();\n"
            + "\t\t$('#c_off').toggle();\n"
            + "\t});\n"
            + "\t\n"
            + "\t$('#favor_trigger').click(function(){\n"
            + "\t\t\tadd_to_favor(107307,2);\n"
            + "\t\t});\n"
            + "});\n"
            + "function ask_too(qid, ask){\n"
            + "\tajax_post(\"/action/question/ask_too?id=\"+qid+\"&ask=\"+ask,\"\",function(html){\n"
            + "\t\tjson = eval('('+html+')');\n"
            + "\t\tif(json.asker_count >= 0){\n"
            + "\t\t\t$('#c_asker_count').html(json.asker_count);\n"
            + "\t\t\tif(json.ask_mode)\n"
            + "\t\t\t\t$('#RQuestionAction').html(\"<span class='rect'>已问同一问题 | <a href='javascript:ask_too(107307,false)'>取消？</a></span>\");\n"
            + "\t\t\telse\n"
            + "\t\t\t\t$('#RQuestionAction').html(\"<a href='javascript:ask_too(107307,true)' class='rndbutton'><span>我想问同样的问题</span></a>\");\t\t\t\n"
            + "\t\t}\n"
            + "\t\telse{\n"
            + "\t\t\t$('#RQuestionAction').poshytip({\n"
            + "\t\t\t\tclassName: 'tip-yellowsimple',\n"
            + "\t\t\t\tcontent: json.msg,\n"
            + "\t\t\t\tshowOn: 'none',\n"
            + "\t\t\t\talignTo: 'target',\n"
            + "\t\t\t\talignX: 'center',\n"
            + "\t\t\t\talignY: 'top',\n"
            + "\t\t\t\toffsetY: 6\n"
            + "\t\t\t});\n"
            + "\t\t\t$('#RQuestionAction').poshytip('show');\n"
            + "\t\t\tvar t = setTimeout(function(){\n"
            + "\t\t\t\tclearTimeout(t);\n"
            + "\t\t\t\t$('#RQuestionAction').poshytip('destroy');\n"
            + "\t\t\t},4000);\n"
            + "\t\t}\n"
            + "\t});\n"
            + "}\n"
            + "function delete_q(qid){\n"
            + "\tif(!confirm(\"您确认要删除此问题吗，删除的数据不可恢复？\"))\n"
            + "\t\treturn ;\n"
            + "\t\tajax_post(\"/action/question/delete?id=\"+qid+\"&hash=-500641190\",\"\",function(html){\n"
            + "\t\tif(html.length>0)\n"
            + "\t\t\talert(html);\n"
            + "\t\telse{\n"
            + "\t\t\tlocation.href = \"/question\";\n"
            + "\t\t}\n"
            + "\t});\n"
            + "}\n"
            + "function edit_answer(aid){\n"
            + "\tlocation.href=\"/question/edit_answer?id=\"+aid;\n"
            + "}\n"
            + "function delete_answer(aid,hash){\n"
            + "\tif(!confirm(\"您确认要删除此答案吗，删除的数据不可恢复？\"))\n"
            + "\t\treturn ;\n"
            + "\tajax_post(\"/action/question/delete_answer?id=\"+aid+\"&hash=\"+hash,\"\",function(html){\n"
            + "\t\tif(html.length>0)\n"
            + "\t\t\talert(html);\n"
            + "\t\telse{\n"
            + "\t\t\t$('#answer_'+aid).fadeOut();\n"
            + "\t\t}\n"
            + "\t});\n"
            + "}\n"
            + "function delete_post_reply(aid,hash){\n"
            + "\tif(!confirm(\"您确认要删除此评论吗，删除的数据不可恢复？\"))\n"
            + "\t\treturn ;\n"
            + "\tajax_post(\"/action/question/delete_answer?id=\"+aid+\"&hash=\"+hash,\"\",function(html){\n"
            + "\t\tif(html.length>0)\n"
            + "\t\t\talert(html);\n"
            + "\t\telse{\n"
            + "\t\t\t$('#PostReply_'+aid).fadeOut();\n"
            + "\t\t}\n"
            + "\t});\n"
            + "}\n"
            + "function delete_q_rpl(qid, rid){\n"
            + "\tif(!confirm(\"删除补充说明会被扣威望值，是否继续？\"))\n"
            + "\t\treturn ;\n"
            + "\tajax_post(\"/action/question/delete_detail?id=\"+rid,\"\",function(html){\n"
            + "\t\tif(html.length>0)\n"
            + "\t\t\talert(html);\n"
            + "\t\telse\n"
            + "\t\t\t$(\"#q_reply_\"+rid).fadeOut();\n"
            + "\t});\n"
            + "}\n"
            + "function close_tip(tid){$('#'+tid).poshytip('destroy');}\n"
            + "//答案投票\n"
            + "function vote_answer(qid, vote_up, need_confirm){\n"
            + "\tif(need_confirm && !vote_up){\n"
            + "\t\tif(!$('#a_post_votedown_' + qid).hasClass('bold')){\n"
            + "\t\t\tvar vote_down_confirm_msg = \"<p>此操作将会扣掉你1个积分，是否继续？</p><p style='margin-top:10px;'><a href='javascript:vote_answer(\"+qid+\",false,false)' class='rbtn' style='margin:0 10px 0 50px;'><span>确定</span></a><a href=\\\"javascript:close_tip('a_post_votedown_\" + qid +\"')\\\" class='rbtn'><span>取消</span></a></p>\";\t\t\t\n"
            + "\t\t\t$('#a_post_votedown_' + qid).poshytip({\n"
            + "\t\t\t\tclassName: 'tip-yellowsimple',\n"
            + "\t\t\t\tcontent: vote_down_confirm_msg,\n"
            + "\t\t\t\tshowOn: 'none',\n"
            + "\t\t\t\tslide: false,\n"
            + "\t\t\t\tfade: false,\n"
            + "\t\t\t\talignTo: 'target',\n"
            + "\t\t\t\talignX: 'center',\n"
            + "\t\t\t\toffsetY: 8\n"
            + "\t\t\t});\n"
            + "\t\t\t$('#a_post_votedown_' + qid).poshytip('show');\n"
            + "\t\t\treturn;\n"
            + "\t\t}\n"
            + "\t}\n"
            + "\tif(!need_confirm){\n"
            + "\t\t$('#a_post_votedown_' + qid).poshytip('destroy');\n"
            + "\t}\n"
            + "\tajax_post(\"/action/question/vote_answer?id=\"+qid+\"&vote=\"+vote_up+\"&user=190591\",\"\",function(data){\n"
            + "\t\tjson = eval('('+data+')');\n"
            + "\t\tif(json.msg){\n"
            + "\t\t\tvar aid = vote_up?\"a_post_voteup_\":\"a_post_votedown_\";\n"
            + "\t\t\taid += qid;\n"
            + "\t\t\t$('#'+aid).poshytip({\n"
            + "\t\t\t\tclassName: 'tip-yellowsimple',\n"
            + "\t\t\t\tcontent: json.msg,\n"
            + "\t\t\t\tshowOn: 'none',\n"
            + "\t\t\t\talignTo: 'target',\n"
            + "\t\t\t\talignX: 'center',\n"
            + "\t\t\t\talignY: 'top',\n"
            + "\t\t\t\toffsetY: 6\n"
            + "\t\t\t});\n"
            + "\t\t\t$('#'+aid).poshytip('show');\n"
            + "\t\t\tvar t = setTimeout(function(){\n"
            + "\t\t\t\tclearTimeout(t);\n"
            + "\t\t\t\t$('#'+aid).poshytip('destroy');\n"
            + "\t\t\t},2000);\n"
            + "\t\t\t//jQuery.fancybox(\"<div class='error_box'>\"+json.msg+\"</div>\");\n"
            + "\t\t}\n"
            + "\t\telse{\n"
            + "\t\t\tif(vote_up){\n"
            + "\t\t\t\t$('#post_voteup_'+qid).html('('+json.vote+')');\n"
            + "\t\t\t\t$('#a_post_voteup_'+qid).toggleClass('bold');\n"
            + "\t\t\t}\n"
            + "\t\t\telse{\n"
            + "\t\t\t\t$('#post_votedown_'+qid).html('('+json.vote+')');\n"
            + "\t\t\t\t$('#a_post_votedown_'+qid).toggleClass('bold');\n"
            + "\t\t\t}\n"
            + "\t\t}\n"
            + "\t});\n"
            + "}\n"
            + "//问题投票 \n"
            + "$('#vote_up').click(function(){\n"
            + "\tif(this.clickTimeout){\n"
            + "        // 双击\n"
            + "        clearTimeout(this.clickTimeout);\n"
            + "        this.clickTimeout = null;\n"
            + "\t\talert(\"不用那么费劲啦，点击一下就够了:)\");\n"
            + "    }\n"
            + "    else{\n"
            + "        // 单击\n"
            + "        var elem = this;\n"
            + "        this.clickTimeout = setTimeout(function(){\n"
            + "            // 执行点击动作\n"
            + "            elem.clickTimeout = null;\n"
            + "\t\t\tvote_question(107307,true, true);\n"
            + "        }, 250);\n"
            + "    }\n"
            + "    //阻止链接onclick时的默认行为\n"
            + "    return false;\n"
            + "});\n"
            + "$('#vote_down').click(function(){\n"
            + "\tvote_question(107307,false, true);\n"
            + "\t/*\n"
            + "\tif(this.clickTimeout){\n"
            + "        // 双击\n"
            + "        clearTimeout(this.clickTimeout);\n"
            + "        this.clickTimeout = null;\n"
            + "\t\talert(\"不用那么费劲啦，点击一下就够了:)\");\n"
            + "    }\n"
            + "    else{\n"
            + "        // 单击\n"
            + "        var elem = this;\n"
            + "        this.clickTimeout = setTimeout(function(){\n"
            + "            // 执行点击动作\n"
            + "            elem.clickTimeout = null;\n"
            + "\t\t\tvote_question(107307,false, true);\n"
            + "        }, 250);\n"
            + "    }\n"
            + "    //阻止链接onclick时的默认行为\n"
            + "\t*/\n"
            + "    return false;\n"
            + "});\n"
            + "function vote_question(qid, vote_up, need_confirm){\n"
            + "\tif(need_confirm && !vote_up){\n"
            + "\t\tif($('#Vote #vote_down').hasClass('ask_vote_down')){\n"
            + "\t\t\tvar vote_down_confirm_msg = \"<p>踩问题将会扣掉你1个积分，是否继续？</p><p style='margin-top:10px;'><a href='javascript:vote_question(107307,false,false)' class='rbtn' style='margin-right:10px;'><span>确定</span></a><a href=\\\"javascript:close_tip('vote_down')\\\" class='rbtn'><span>取消</span></a></p>\";\n"
            + "\t\t\t$('#Vote #vote_down').poshytip({\n"
            + "\t\t\t\tclassName: 'tip-yellowsimple',\n"
            + "\t\t\t\tcontent: vote_down_confirm_msg,\n"
            + "\t\t\t\tslide: false,\n"
            + "\t\t\t\tfade: false,\n"
            + "\t\t\t\tshowOn: 'none',\n"
            + "\t\t\t\talignTo: 'target',\n"
            + "\t\t\t\talignX: 'inner-right',\n"
            + "\t\t\t\talignY: 'bottom',\n"
            + "\t\t\t\toffsetX: -30,\n"
            + "\t\t\t\toffsetY: 15\n"
            + "\t\t\t});\n"
            + "\t\t\t$('#Vote #vote_down').poshytip('show');\n"
            + "\t\t\treturn;\n"
            + "\t\t}\n"
            + "\t}\n"
            + "\tif(!need_confirm){\n"
            + "\t\t$('#Vote #vote_down').poshytip('destroy');\n"
            + "\t}\n"
            + "\tajax_post(\"/action/question/vote?user=190591&id=\"+qid+\"&vote=\"+vote_up,\"\",function(data){\n"
            + "\t\tjson = eval('('+data+')');\n"
            + "\t\tif(json.msg){\n"
            + "\t\t\tvar aid = vote_up?\"vote_up\":\"vote_down\";\n"
            + "\t\t\t$('#'+aid).poshytip({\n"
            + "\t\t\t\tclassName: 'tip-yellowsimple',\n"
            + "\t\t\t\tcontent: json.msg,\n"
            + "\t\t\t\tshowOn: 'none',\n"
            + "\t\t\t\talignTo: 'target',\n"
            + "\t\t\t\talignX: 'inner-right',\n"
            + "\t\t\t\talignY: 'bottom',\n"
            + "\t\t\t\toffsetX: 5,\n"
            + "\t\t\t\toffsetX: -35\n"
            + "\t\t\t});\n"
            + "\t\t\t$('#'+aid).poshytip('show');\n"
            + "\t\t\tvar t = setTimeout(function(){\n"
            + "\t\t\t\tclearTimeout(t);\n"
            + "\t\t\t\t$('#'+aid).poshytip('destroy');\n"
            + "\t\t\t},2000);\n"
            + "\t\t}\n"
            + "\t\telse{\n"
            + "\t\t\t\t\t\tif(vote_up){\n"
            + "\t\t\t\t$('#Vote .vote-up-count').html(json.vote_up);\n"
            + "\t\t\t\t$('#Vote #vote_up').toggleClass('ask_vote_up');\n"
            + "\t\t\t\t$('#Vote #vote_up').toggleClass('ask_vote_uped');\n"
            + "\t\t\t}\n"
            + "\t\t\telse{\n"
            + "\t\t\t\t$('#Vote .vote-down-count').html(json.vote_down);\n"
            + "\t\t\t\t$('#Vote #vote_down').toggleClass('ask_vote_down');\n"
            + "\t\t\t\t$('#Vote #vote_down').toggleClass('ask_vote_downed');\n"
            + "\t\t\t}\n"
            + "\t\t}\n"
            + "\t});\n"
            + "}\n"
            + "//评论答案\n"
            + "function reply_to_post(postid,uid){\n"
            + "\tpopup(\"/action/ajax/reply_to_post?id=\" + postid + \"&refer=\"+uid);\n"
            + "\t}\n"
            + "\n"
            + "function show_rp_next(postid,current,total){\n"
            + "\tif(current < total){\n"
            + "\t\tvar next_page = current + 1;\n"
            + "    \tvar url = \"/question/post_replies?answer=\"+postid+\"&rp=\"+next_page;\n"
            + "    \tajax_post(url,\"\",function(html){\n"
            + "    \t\t$('#PostReplies_'+postid).html(html);\n"
            + "\t\t\t\taddRepliesHoverEvent();\n"
            + "\t\t\t});\n"
            + "\t}\n"
            + "}\n"
            + "\n"
            + "function addRepliesHoverEvent(){\n"
            + "\t$(\"li [id ^= 'PostReply']\").hover(function(){\n"
            + "\t\t$(this).addClass(\"hover\");\n"
            + "\t},function(){\n"
            + "\t\t$(this).removeClass(\"hover\");\n"
            + "\t});\n"
            + "}\n"
            + "\n"
            + "function show_rp_prev(postid,current,total){\n"
            + "\tif(current > 1){\n"
            + "\t\tvar next_page = current - 1;\n"
            + "    \tvar url = \"/question/post_replies?answer=\"+postid+\"&rp=\"+next_page;\n"
            + "    \tajax_post(url,\"\",function(html){\n"
            + "    \t\t$('#PostReplies_'+postid).html(html);\n"
            + "\t\t\taddRepliesHoverEvent();\n"
            + "    \t});\n"
            + "\t}\n"
            + "}\n"
            + "function mark_as_top(qid, as_top) {\n"
            + "\tvar args = \"id=\"+qid+\"&top=\"+as_top;\n"
            + "\tajax_post(\"/action/question/mark_as_top\",args,function(html){\n"
            + "\t\talert(html);\n"
            + "\t});\n"
            + "}\n"
            + "function mark_as_best(postid, is_best){\n"
            + "\tvar args = \"id=\"+postid+\"&best=\"+(is_best?1:0);\n"
            + "\tajax_post(\"/action/question/mark_as_best\",args,function(html){\n"
            + "    \tif(html.length>0){\n"
            + "    \t\t$('#best_answer_'+postid).poshytip({\n"
            + "    \t\t\tclassName: 'tip-yellowsimple',\n"
            + "    \t\t\tcontent: html,\n"
            + "    \t\t\tshowOn: 'none',\n"
            + "    \t\t\talignTo: 'target',\n"
            + "    \t\t\talignX: 'center',\n"
            + "    \t\t\talignY: 'top',\n"
            + "    \t\t\toffsetY: 6\n"
            + "    \t\t});\n"
            + "    \t\t$('#best_answer_'+postid).poshytip('show');\n"
            + "    \t\tvar t = setTimeout(function(){\n"
            + "    \t\t\tclearTimeout(t);\n"
            + "    \t\t\t$('#best_answer_'+postid).poshytip('destroy');\n"
            + "    \t\t},2000);\n"
            + "    \t}\n"
            + "    \telse{\n"
            + "\t\t\tif(is_best)\n"
            + "\t\t\t\t$('#answer_'+postid).addClass('Best');\n"
            + "\t\t\telse\n"
            + "\t\t\t\t$('#answer_'+postid).removeClass('Best');\n"
            + "    \t}\n"
            + "\t});\n"
            + "}\n"
            + "function edit_tags(qid){\n"
            + "\tpopup(\"/question/edit_tags?question=\"+qid);\n"
            + "}\n"
            + "function edit_catalogs(qid){\n"
            + "\tpopup(\"/admin/catalog/set-catalogs?parent=1&type=2&id=\"+qid);\n"
            + "}\n"
            + "function event_apply(event_id){\t\n"
            + "\tpopup(\"/action/ajax/event_apply\",\"id=\"+event_id);\n"
            + "}\n"
            + "function cancel_apply(event_id){\n"
            + "\tif(confirm(\"您确认要取消参加此次活动吗？\")){\n"
            + "\t\tajax_post(\"/action/event/cancel\",\"event=\"+event_id,function(html){\n"
            + "\t\t\tif(html.length>0)\n"
            + "\t\t\t\talert(html);\n"
            + "\t\t\telse\n"
            + "\t\t\t\talert('已取消参加此次活动，感谢您的支持:)');\n"
            + "\t\t});\n"
            + "\t}\n"
            + "}\n"
            + "\n"
            + "var favor_ok = \"<p class='favor_ok'>已成功添加到收藏夹<br><br> <a href='http://my.oschina.net/flashsword/favorites?type=$DAISY_OBJ_TYPE'>我的收藏夹</a> | <a href='javascript:close_favor()'>关闭</a></p>\";\n"
            + "function delete_favor(obi_id, obj_type){\n"
            + "\tif(!confirm('确定取消收藏？')) return;\n"
            + "\t$.post(\"/action/favorite/cancel?type=\"+obj_type+\"&id=\"+obi_id+\"&user=190591\",function(html){\n"
            + "\t\t$('#favor_trigger').parent('div').removeClass('ask_collected').addClass('ask_collect');\n"
            + "\t\t$('#favor_trigger').attr('title','添加到收藏');\n"
            + "    \t$('#p_favor_count').html(html);\n"
            + "\t\t$('#favor_trigger').unbind('click');\n"
            + "\t\t$('#favor_trigger').click(function(){add_to_favor(107307,2);});\n"
            + "\t});\n"
            + "}\n"
            + "function add_to_favor(obj_id, obj_type){\n"
            + "    var dlg_favor = \"<div id='favor_form'><p>多个标签使用逗号(,)隔开，最多三个</p><form action='/action/favorite/add?user=190591' height='60px' width='200px' method='POST'>\";\n"
            + "\tdlg_favor += \"<input type='hidden' name='id' value='\"+obj_id+\"'/>\";\n"
            + "\tdlg_favor += \"<input type='hidden' name='type' value='\"+obj_type+\"'/>\";\n"
            + "\tdlg_favor += \"<input type='text' name='tags' size='25' class='_favor_input' id='_favor_tags'/>\";\n"
            + "\tdlg_favor += \"<input type='submit' value='收藏' class='_favor_button'/><input type='button' value='取消' onclick='close_favor();' class='_favor_button'/><a id='TagsSwitcher' state='off'>选取标签↓</a></form>\";\n"
            + "\tdlg_favor += \"<div id='MyTags' ></div></div>\";\n"
            + "    $('#favor_trigger').poshytip('destroy');\n"
            + "    $('#favor_trigger').poshytip({\n"
            + "    \tclassName: 'tip-yellowsimple',\n"
            + "    \tcontent: dlg_favor,\n"
            + "    \tshowOn: 'none',\n"
            + "    \talignTo: 'target',\t\n"
            + "\t\talignX: 'inner-right',\n"
            + "\t\talignY: 'bottom',\n"
            + "\t\toffsetX: -20,\n"
            + "\t\toffsetY: 15\n"
            + "    });\n"
            + "    $('#favor_trigger').poshytip('show');\n"
            + "\t$('#_favor_tags').focus();\n"
            + "\t$('#favor_form form').ajaxForm({\n"
            + "\t\tsuccess: function(html) {\n"
            + "\t\t\t$('#favor_trigger').parent('div').removeClass('ask_collect').addClass('ask_collected');\n"
            + "\t\t\t$('#favor_trigger').attr('title','取消收藏');\n"
            + "\t\t\t$('#p_favor_count').html(html);\n"
            + "\t\t\t$('#favor_trigger').unbind('click');\n"
            + "\t\t\t$('#favor_trigger').click(function(){delete_favor(107307,2);});\n"
            + "\t\t\t$('#favor_form').html(favor_ok);\n"
            + "\t\t\tsetTimeout(\"close_favor()\",3000);\n"
            + "\t\t}\n"
            + "\t});\n"
            + "\t$(\"#TagsSwitcher\").one(\"click\",function(){\n"
            + "\t\t//加载标签数据\n"
            + "\t\t$(\"#MyTags\").load('/action/ajax/get_my_tags');\n"
            + "       \t$(\"#MyTags\").toggle();\n"
            + "\t\t$(this).html(\"收起标签↑\");\n"
            + "        $(this).attr(\"state\",'on');\n"
            + "        $(this).click(function(){\n"
            + "        \t$(\"#MyTags\").toggle();\n"
            + "        \tvar state = $(this).attr(\"state\");\n"
            + "        \tif(state=='off'){\n"
            + "        \t\t$(this).html(\"收起标签↑\");\n"
            + "        \t\t$(this).attr(\"state\",'on');\n"
            + "        \t}else{\n"
            + "        \t\t$(this).html(\"选取标签↓\");\n"
            + "        \t\t$(this).attr(\"state\",'off');\n"
            + "        \t}\n"
            + "        });\n"
            + "\t});\n"
            + "\n"
            + "}\n"
            + "function close_favor(elem_id){\n"
            + "    $('#favor_trigger').poshytip('destroy');\n"
            + "}\n"
            + "function setTag(tv){\n"
            + "\tvar t = $(\"._favor_input\");\n"
            + "\tvar value = t.val();\n"
            + "\tif(value!=\"\")\n"
            + "\t\tt.val(value+\",\"+tv);\n"
            + "\telse\n"
            + "\t\tt.val(tv)\n"
            + "}\n"
            + "//-->\n"
            + "</script><div class='clear'></div></div>\n"
            + "\t<div id=\"OSC_Footer\" class='CenterDiv'><style>\n"
            + ".oscapp {text-align:left; width:220px;}\n"
            + ".oscapp span {float:left;width:140px;}\n"
            + ".oscapp a {float:left;text-indent:-9999em;width:16px;margin-left:8px;}\n"
            + ".oscapp a.android {background:url('/img/android.gif') no-repeat left center;}\n"
            + ".oscapp a.iphone {background:url('/img/iphone.gif') no-repeat left center;}\n"
            + ".oscapp a.wp7 {background:url('/img/wp7.gif') no-repeat left center;}\n"
            + "</style>\n"
            + "<table width='100%'><tr>\n"
            + "<td align='left'>&copy; 开源中国(OsChina.NET) | <a href=\"http://www.oschina.net/home/about\">关于我们</a> | <a href=\"mailto:oschina.net@gmail.com\">广告联系</a> | <a href=\"http://weibo.com/oschina2010\" target=\"_blank\">@新浪微博</a> | <a href=\"http://m.oschina.net/\">开源中国手机版</a> | <a href='http://www.miitbeian.gov.cn/' target='_blank' style='color:#737573;text-decoration:none;'>粤ICP备12009483号-3</a></td>\n"
            + "<td class='oscapp'>\n" + "\t<span>开源中国手机客户端：</span>\n"
            + "\t<a href=\"http://www.oschina.net/app\" class='android' title='Android客户端'>Android</a>\n"
            + "\t<a href=\"http://www.oschina.net/app\" class='iphone' title='iPhone 客户端'>iPhone</a>\n"
            + "\t<a href=\"http://www.oschina.net/app\" class='wp7' title='Windows Phone 客户端'>WP7</a>\n" + "</td>\n"
            + "</tr>\n" + "</table>\n" + "<script type='text/javascript'>\n" + "<!--\n"
            + "if (top.location != self.location)top.location=self.location;\n" + "//-->\n" + "</script></div>\n"
            + "</div>\n" + "</body>\n" + "</html>\n"
            + "<!-- Generated by OsChina.NET (init:1[ms],page:43[ms],ip:60.55.11.77) -->";

    @Test
    public void test() {
        String text = "\n"
                + "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n"
                + "<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"zh-CN\" dir=\"ltr\">\n"
                + "  <head>\n"
                + "    <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"/>\n"
                + "    <title>jsoup 解析页面商品信息 -  - ITeye技术网站</title>\n"
                + "    <meta name=\"description\" content=\"今天用了jsoup 解析页面商品信息，感觉比用xpath获取信息准确多了     下面就记录一下：  一、首先去 http://jsoup.org/download 下载jsoup的jar包。     二、下面记录下相关代码：              Document doc = Jsoup.connect(url).get();    //将htm转换成Document类型数据结构        ...\" />\n"
                + "    <meta name=\"keywords\" content=\" jsoup 解析页面商品信息\" />\n"
                + "    <link rel=\"shortcut icon\" href=\"/images/favicon.ico\" type=\"image/x-icon\" />\n"
                + "    <link rel=\"search\" type=\"application/opensearchdescription+xml\" href=\"/open_search.xml\" title=\"ITeye\" />\n"
                + "    <link href=\"/rss\" rel=\"alternate\" title=\"\" type=\"application/rss+xml\" />\n"
                + "    <link href=\"http://www.iteye.com/stylesheets/blog.css?1365750118\" media=\"screen\" rel=\"stylesheet\" type=\"text/css\" />\n"
                + "<link href=\"http://www.iteye.com/stylesheets/themes/blog/blue.css?1326191326\" media=\"screen\" rel=\"stylesheet\" type=\"text/css\" />\n"
                + "    <script src=\"http://www.iteye.com/javascripts/application.js?1358214518\" type=\"text/javascript\"></script>\n"
                + "    <script type=\"text/javascript\">\n"
                + "\n"
                + "  var _gaq = _gaq || [];\n"
                + "  _gaq.push(['_setAccount', 'UA-535605-1']);\n"
                + "  _gaq.push(['_setDomainName', 'iteye.com']);\n"
                + "  _gaq.push(['_trackPageview']);\n"
                + "\n"
                + "  (function() {\n"
                + "    var ga = document.createElement('script'); ga.type = 'text/javascript'; ga.async = true;\n"
                + "    ga.src = ('https:' == document.location.protocol ? 'https://ssl' : 'http://www') + '.google-analytics.com/ga.js';\n"
                + "    var s = document.getElementsByTagName('script')[0]; s.parentNode.insertBefore(ga, s);\n"
                + "  })();\n"
                + "\n"
                + "</script>\n"
                + "\n"
                + "\n"
                + "      <link href=\"http://www.iteye.com/javascripts/syntaxhighlighter/SyntaxHighlighter.css?1348819953\" media=\"screen\" rel=\"stylesheet\" type=\"text/css\" />\n"
                + "  <script src=\"http://www.iteye.com/javascripts/syntaxhighlighter/shCoreCommon.js?1325907333\" type=\"text/javascript\"></script>\n"
                + "<script src=\"http://www.iteye.com/javascripts/hotkey.js?1324994303\" type=\"text/javascript\"></script>\n"
                + "  <script src=\"http://www.iteye.com/javascripts/code_favorites.js?1358214518\" type=\"text/javascript\"></script>\n"
                + "<script src=\"http://www.iteye.com/javascripts/weiboshare.js?1324994303\" type=\"text/javascript\"></script>\n"
                + "  <link href=\"http://www.iteye.com/javascripts/editor/css/ui.css?1324994303\" media=\"screen\" rel=\"stylesheet\" type=\"text/css\" />\n"
                + "  <script src=\"http://www.iteye.com/javascripts/editor/compress.js?1358129160\" type=\"text/javascript\"></script>\n"
                + "  </head>\n"
                + "  <body>\n"
                + "    <div id=\"header\">\n"
                + "      <div id=\"blog_site_nav\">\n"
                + "  <a href=\"http://www.iteye.com/\" class=\"homepage\">首页</a>\n"
                + "  <a href=\"http://www.iteye.com/news\">资讯</a>\n"
                + "  <a href=\"http://www.iteye.com/magazines\">精华</a>\n"
                + "  <a href=\"http://www.iteye.com/forums\">论坛</a>\n"
                + "  <a href=\"http://www.iteye.com/ask\">问答</a>\n"
                + "  <a href=\"http://www.iteye.com/blogs\">博客</a>\n"
                + "  <a href=\"http://www.iteye.com/blogs/subjects\">专栏</a>\n"
                + "  <a href=\"http://www.iteye.com/groups\">群组</a>\n"
                + "  <a href=\"#\" onclick=\"return false;\" id=\"msna\"><u>更多</u> <small>▼</small></a>\n"
                + "  <div class=\"quick_menu\" style=\"display:none;\">\n"
                + "    <a target=\"_blank\" href=\"http://job.iteye.com/iteye\">招聘</a>\n"
                + "    <a href=\"http://www.iteye.com/search\">搜索</a>\n"
                + "  </div>\n"
                + "</div>\n"
                + "\n"
                + "      <div id=\"user_nav\">\n"
                + "  \n"
                + "        <a href=\"http://flashsword20.iteye.com\" title=\"查看我的博客首页\" class=\"welcome\">欢迎flashsword20</a>\n"
                + "    <a id=\"notifications_count\" href=\"http://my.iteye.com/notifications\">0</a>\n"
                + "    \n"
                + "      <a href=\"http://my.iteye.com/messages\" title=\"你有新的站内短信\"><img alt=\"Newpm\" src=\"http://www.iteye.com/images/newpm.gif?1324994303\" />收件箱(3)</a>\n"
                + "    \n"
                + "    <a href=\"http://my.iteye.com\" title=\"我的应用首页\">我的应用</a>\n"
                + "    <div class=\"quick_menu\" style=\"display:none;\">\n"
                + "      <a href=\"http://my.iteye.com/feed\" title=\"我关注的好友动态消息\">我的关注</a>\n"
                + "      <a href=\"http://my.iteye.com/mygroup\" title=\"我加入的群组最新话题\">我的群组</a>\n"
                + "      <a href=\"http://my.iteye.com/myresume\" title=\"我的个人简历\">我的简历</a>\n"
                + "      <a href=\"http://my.iteye.com/admin/album\" title=\"我的个人简历\">我的相册</a>\n"
                + "      <a href=\"http://my.iteye.com/admin/link\" title=\"我收藏的网络资源链接\">我的收藏</a>\n"
                + "      <a href=\"http://my.iteye.com/admin/code\" title=\"我收藏的代码\">我的代码</a>\n"
                + "      <a href=\"http://my.iteye.com/admin/weibo\" title=\"用微博发表简短的话题\">我的微博</a>\n"
                + "    </div>\n"
                + "    <a href=\"http://flashsword20.iteye.com/admin\" title=\"管理我的博客\">我的博客</a>\n"
                + "    <a href=\"http://my.iteye.com/profile\" title=\"修改我的个人设置\">设置</a>\n"
                + "    <a href=\"/logout\" class=\"nobg\" onclick=\"var f = document.createElement('form'); f.style.display = 'none'; this.parentNode.appendChild(f); f.method = 'POST'; f.action = this.href;var m = document.createElement('input'); m.setAttribute('type', 'hidden'); m.setAttribute('name', '_method'); m.setAttribute('value', 'put'); f.appendChild(m);var s = document.createElement('input'); s.setAttribute('type', 'hidden'); s.setAttribute('name', 'authenticity_token'); s.setAttribute('value', 'TDmn/IsWi1Aj4CXKfdMKZZzALz6jbRU/Biw0/QHnsVw='); f.appendChild(s);f.submit();return false;\">退出</a>\n"
                + "  </div>\n"
                + "\n"
                + "    </div>\n"
                + "\n"
                + "    <div id=\"page\">\n"
                + "      <div id=\"branding\" class=\"clearfix\">\n"
                + "        <div id=\"blog_name\">\n"
                + "          <h1><a href=\"/\">masong1987</a></h1>\n"
                + "        </div>\n"
                + "        <div id='fd'></div>\n"
                + "        <div id=\"blog_navbar\">\n"
                + "          <ul>\n"
                + "            <li class='blog_navbar_for'><a href=\"http://masong1987.iteye.com\"><strong>博客</strong></a></li>\n"
                + "            <li ><a href=\"/weibo\">微博</a></li>\n"
                + "            <li ><a href=\"/album\">相册</a></li>\n"
                + "            <li ><a href=\"/link\">收藏</a></li>\n"
                + "            <li ><a href=\"/blog/guest_book\">留言</a></li>\n"
                + "            <li ><a href=\"/blog/profile\">关于我</a></li>\n"
                + "          </ul>\n"
                + "    \n"
                + "          <div class=\"search\">\n"
                + "            <form action=\"/blog/search\" method=\"get\">\n"
                + "              <input class=\"search_text\" id=\"query\" name=\"query\" style=\"margin-left: 10px;width: 110px;\" type=\"text\" value=\"\" />\n"
                + "              <input class=\"submit_search\" type=\"submit\" value=\"\" />\n"
                + "            </form>\n"
                + "          </div> \n"
                + "          <div id=\"fd\"></div>         \n"
                + "        </div>\n"
                + "      </div>\n"
                + "      \n"
                + "      <div id=\"content\" class=\"clearfix\">\n"
                + "        <div id=\"main\">\n"
                + "          \n"
                + "\n"
                + "\n"
                + "\n"
                + "          \n"
                + "\n"
                + "\n"
                + "<div class=\"blog_main\">\n"
                + "  <div class=\"blog_title\">\n"
                + "    <h3>\n"
                + "      <a href=\"/blog/1191016\">jsoup 解析页面商品信息</a>\n"
                + "      <em class=\"actions\">      </em>\n"
                + "    </h3>\n"
                + "    <ul class='blog_categories'><strong>博客分类：</strong> <li><a href=\"/category/182324\">爬虫</a></li> </ul>\n"
                + "        <div class='news_tag'>&nbsp;</div>\n"
                + "  </div>\n"
                + "\n"
                + "  <div id=\"blog_content\" class=\"blog_content\">\n"
                + "    <p>今天用了jsoup 解析页面商品信息，感觉比用xpath获取信息准确多了</p>\n"
                + "<p> </p>\n"
                + "<p>下面就记录一下：</p>\n"
                + "<p>一、首先去 <a href=\"http://jsoup.org/download\">http://jsoup.org/download</a> 下载jsoup的jar包。</p>\n"
                + "<p> </p>\n"
                + "<p>二、下面记录下相关代码：</p>\n"
                + "<p> </p>\n"
                + "<p> </p>\n"
                + "<p>      Document doc = Jsoup.connect(url).get();    //将htm转换成Document类型数据结构</p>\n"
                + "<p> <br>      doc.select(\"div:has(div) div#spec-n1:has(img) img\").first().attr(\"src\"));    //查找div下含有div的标签</p>\n"
                + "<p>      </p>\n"
                + "<p>      并且 div的id='spec-n1'，此div第一个img标签，img里属性是src的值。</p>\n"
                + "<p> </p>\n"
                + "<p>      doc.select(\"div:has(div) div.crumb:has(a) a:eq(4)\").text();    //查找class='crumb'的div下第4个a标签</p>\n"
                + "<p>      下的值。</p>\n"
                + "<p> </p>\n"
                + "<p>      doc.select(\"div:has(div) div#name:has(h1)\").text();     //查找id='name'的div下的h1标签的值。</p>\n"
                + "<p> </p>\n"
                + "<p>      doc.select(\"tbody:has(tr) td.tdTitle:contains(品牌) + td\").text();     //查找class='tdTitle'的td标签里</p>\n"
                + "<p> </p>\n"
                + "<p>      含有‘品牌’td的下一个td标签中内容。</p>\n"
                + "<p> </p>\n"
                + "<p>      doc.select(\"script[type=text/javascript]:not([src~=[a-zA-Z0-9./\\\\s]+)\");     //查找含有此&lt;script </p>\n"
                + "<p> </p>\n"
                + "<p>      type=\"text/javascript\"&gt;……&lt;/script&gt;内容，不含有script标签中有src属性的script，如：</p>\n"
                + "<p> </p>\n"
                + "<p>      &lt;script src=\"url\" type=\"text/javascript\"&gt;&lt;/script&gt;。</p>\n"
                + "  </div>\n"
                + "\n"
                + "  \n"
                + "\n"
                + "\n"
                + "  <IFRAME SRC=\"/iframe_ggbd/794\" SCROLLING=no WIDTH=468 HEIGHT=60 FRAMEBORDER=0></IFRAME>\n"
                + "  \n"
                + "  <div id=\"bottoms\" class=\"clearfix\">\n"
                + "    \n"
                + "    <div id=\"share_weibo\">分享到：\n"
                + "      <a data-type='sina' href=\"javascript:;\" title=\"分享到新浪微博\"><img src=\"/images/sina.jpg\"></a>\n"
                + "      <a data-type='qq' href=\"javascript:;\" title=\"分享到腾讯微博\"><img src=\"/images/tec.jpg\"></a>\n"
                + "    </div>\n"
                + "  </div>\n"
                + "\n"
                + "  <div class=\"blog_nav\">\n"
                + "    <div class=\"pre_next\">\n"
                + "      <a href=\"/blog/1310327\" class=\"next\" title=\"ibatis中书写SQL语句时使用in遇到的问题\">ibatis中书写SQL语句时使用in遇到的问题</a>\n"
                + "      |\n"
                + "      <a href=\"/blog/1189699\" class=\"pre\" title=\"尚未备份数据库 &quot;***&quot; 的日志尾部。如果该日志包含您不希望丢失的工作，请使用 BACKUP LOG WITH NORECOVERY 备份该日志。请使用 RE\">尚未备份数据库 &quot;***&quot; 的日志尾部。如果该 ...</a>\n"
                + "    </div>\n"
                + "  </div>\n"
                + "  <div class=\"blog_bottom\">\n"
                + "    <ul>\n"
                + "      <li>2011-10-12 18:52</li>\n"
                + "      <li>浏览 692</li>\n"
                + "      <li><a href=\"#comments\">评论(0)</a></li>\n"
                + "      \n"
                + "      \n"
                + "        <li><a href='/admin/link?user_favorite%5Btitle%5D=jsoup+%E8%A7%A3%E6%9E%90%E9%A1%B5%E9%9D%A2%E5%95%86%E5%93%81%E4%BF%A1%E6%81%AF&amp;user_favorite%5Burl%5D=http%3A%2F%2Fmasong1987.iteye.com%2Fblog%2F1191016' target='_blank' class='favorite' onclick=\"$$('.favorite_form_spinner')[0].show();new Ajax.Request('/admin/link/new_xhr?user_favorite%5Btitle%5D=jsoup+%E8%A7%A3%E6%9E%90%E9%A1%B5%E9%9D%A2%E5%95%86%E5%93%81%E4%BF%A1%E6%81%AF&amp;user_favorite%5Burl%5D=http%3A%2F%2Fmasong1987.iteye.com%2Fblog%2F1191016', {method: 'get', onSuccess: function(response){$(document.getElementsByTagName('body')[0]).insert({bottom:response.responseText});$$('.favorite_form_spinner')[0].hide();}});return false;\">收藏</a><img alt=\"Spinner\" class=\"favorite_form_spinner\" src=\"http://www.iteye.com/images/spinner.gif?1324994303\" style=\"vertical-align:bottom;margin-left:7px;display:none;\" /></li>\n"
                + "      \n"
                + "      <li>分类:<a href=\"http://www.iteye.com/blogs/category/opensource\">开源软件</a></li>      \n"
                + "      <li class='last'><a href=\"http://www.iteye.com/wiki/blog/1191016\" target=\"_blank\" class=\"more\">相关推荐</a></li>\n"
                + "    </ul>\n"
                + "  </div>\n"
                + "\n"
                + "  <div class=\"blog_comment\">\n"
                + "    <h5>评论</h5>\n"
                + "    <a id=\"comments\" name=\"comments\"></a>\n"
                + "    \n"
                + "    \n"
                + "    \n"
                + "  </div>\n"
                + "\n"
                + "  <div class=\"blog_comment\">\n"
                + "    <h5>发表评论</h5>\n"
                + "            <form action=\"/blog/1191016\" id=\"comment_form\" method=\"post\" onsubmit=\"return false;\"><div style=\"margin:0;padding:0;display:inline\"><input name=\"authenticity_token\" type=\"hidden\" value=\"TDmn/IsWi1Aj4CXKfdMKZZzALz6jbRU/Biw0/QHnsVw=\" /></div>          \n"
                + "\n"
                + "\n"
                + "  <input type=\"hidden\" id=\"editor_bbcode_flag\"/>\n"
                + "\n"
                + "\n"
                + "\n"
                + "<div id=\"editor_main\"><textarea class=\"validate-richeditor bad-words min-length-5\" cols=\"40\" id=\"editor_body\" name=\"comment[body]\" rows=\"20\" style=\"width: 500px; height: 350px;\"></textarea></div>\n"
                + "\n"
                + "\n"
                + "<script type=\"text/javascript\">\n"
                + "  var editor = new Control.TextArea.Editor(\"editor_body\", \"bbcode\", false);\n"
                + "</script>\n"
                + "\n"
                + "          <p style=\"text-align:right;margin-right:30px;\">(快捷键 Alt+S / Ctrl+Enter) <input class=\"submit\" id=\"quick_reply_button\" name=\"commit\" type=\"submit\" value=\"提交\" /></p>\n"
                + "       </form>\n"
                + "        <script type=\"text/javascript\">\n"
                + "          new HotKey(\"s\",function() {$('quick_reply_button').click();},{altKey: true, ctrlKey: false});\n"
                + "          new HotKey(new Number(13),function() {$('quick_reply_button').click();},{altKey: false, ctrlKey: true});\n"
                + "\n"
                + "          new Validation(\"comment_form\", {immediate: false, onFormValidate: function(result, form){\n"
                + "            if(result) {\n"
                + "              new Ajax.Request('/blog/create_comment/1191016', {\n"
                + "                onFailure:function(response){\n"
                + "                  $('comments').insert({after:response.responseText})\n"
                + "                  form.spinner.hide();\n"
                + "                  Element.scrollTo($('comments'));\n"
                + "                },\n"
                + "                onSuccess:function(response){\n"
                + "                  Element.scrollTo($('comments'));\n"
                + "                  var new_comment = new Element('div', {}).update(response.responseText).firstChild;\n"
                + "                  var comment_id = new_comment.readAttribute('id');\n"
                + "\n"
                + "                  $('comments').insert({after:response.responseText});\n"
                + "                  $('editor_body').value = \"\";\n"
                + "\n"
                + "                  var css_rules = '#' + comment_id + ' pre';\n"
                + "                  highlightNewAddContent(css_rules);\n"
                + "                  processComment();\n"
                + "                  code_favorites_init(css_rules);\n"
                + "                  \n"
                + "                  form.spinner.hide();\n"
                + "                }, parameters:Form.serialize(form)\n"
                + "              });\n"
                + "            }\n"
                + "        }});\n"
                + "        </script>\n"
                + "        </div>\n"
                + "</div>\n"
                + "\n"
                + "\n"
                + "<script type=\"text/javascript\">\n"
                + "  dp.SyntaxHighlighter.HighlightAll('code', true, true);\n"
                + "\n"
                + "  $$('#main .blog_content pre[name=code]').each(function(pre, index){ // blog content\n"
                + "    var post_id = 1191016;\n"
                + "    var location = window.location;\n"
                + "    source_url = location.protocol + \"//\" + location.host + location.pathname + location.search;\n"
                + "    pre.writeAttribute('codeable_id', post_id);\n"
                + "    pre.writeAttribute('codeable_type', \"OschinaBlog\");\n"
                + "    pre.writeAttribute('source_url', source_url);\n"
                + "    pre.writeAttribute('pre_index', index);\n"
                + "    pre.writeAttribute('title', 'jsoup 解析页面商品信息');\n"
                + "  });\n"
                + "\n"
                + "  fix_image_size($$('div.blog_content img'), 700);\n"
                + "\n"
                + "  function processComment() {\n"
                + "    $$('#main .blog_comment > div').each(function(comment){// comment\n"
                + "      var post_id = comment.id.substr(2);\n"
                + "      $$(\"#\"+comment.id+\" pre[name=code]\").each(function(pre, index){\n"
                + "        var location = window.location;\n"
                + "        source_url = location.protocol + \"//\" + location.host + location.pathname + location.search;\n"
                + "        source_url += \"#\" + comment.id;\n"
                + "        pre.writeAttribute('codeable_id', post_id);\n"
                + "        pre.writeAttribute('codeable_type', \"BlogComment\");\n"
                + "        pre.writeAttribute('source_url', source_url);\n"
                + "        pre.writeAttribute('pre_index', index);\n"
                + "        pre.writeAttribute('title', 'jsoup 解析页面商品信息');\n"
                + "      });\n"
                + "    });\n"
                + "  }\n"
                + "\n"
                + "  function quote_comment(id) {\n"
                + "    new Ajax.Request('/editor/quote', {\n"
                + "      parameters: {'id':id, 'type':'BlogComment'},\n"
                + "      onSuccess:function(response){editor.bbcode_editor.textarea.insertAfterSelection(response.responseText);\n"
                + "        Element.scrollTo(editor.bbcode_editor.textarea.element);}\n"
                + "    });\n"
                + "  }\n"
                + "\n"
                + "  code_favorites_init();\n"
                + "  processComment();\n"
                + "  new WeiboShare({share_buttons: $('share_weibo'), img_scope: $('blog_content')});\n"
                + "</script>\n"
                + "\n"
                + "\n"
                + "\n"
                + "\n"
                + "        </div>\n"
                + "\n"
                + "        <div id=\"local\">\n"
                + "          <div class=\"local_top\"></div>\n"
                + "          <div id=\"blog_owner\">\n"
                + "  <div id=\"blog_owner_logo\"><a href='http://masong1987.iteye.com'><img alt=\"masong1987的博客\" class=\"logo\" src=\"http://www.iteye.com/images/user-logo.gif?1324994303\" title=\"masong1987的博客: \" /></a></div>\n"
                + "  <div id=\"blog_owner_name\">masong1987</div>\n"
                + "</div>\n"
                + "\n"
                + "          <div id=\"blog_actions\">\n"
                + "            <ul>\n"
                + "              <li>浏览: 5401 次</li>\n"
                + "              <li>性别: <img alt=\"Icon_minigender_1\" src=\"http://www.iteye.com/images/icon_minigender_1.gif?1324994303\" title=\"男\" /></li>\n"
                + "              <li>来自: 北京</li>\n"
                + "              <li><img src='/images/status/offline.gif'/></li>\n"
                + "              \n"
                + "                <li>\n"
                + "                  <a href=\"http://my.iteye.com/messages/new?message%5Breceiver_name%5D=masong1987\" class=\"message\" title=\"发送站内短信\">发短消息</a>\n"
                + "                  \n"
                + "                    <a href=\"http://my.iteye.com/feed?subscription%5Bsubscribed_user_name%5D=masong1987\" class=\"subscription\" onclick=\"var f = document.createElement('form'); f.style.display = 'none'; this.parentNode.appendChild(f); f.method = 'POST'; f.action = this.href;var s = document.createElement('input'); s.setAttribute('type', 'hidden'); s.setAttribute('name', 'authenticity_token'); s.setAttribute('value', 'TDmn/IsWi1Aj4CXKfdMKZZzALz6jbRU/Biw0/QHnsVw='); f.appendChild(s);f.submit();return false;\">关注</a>\n"
                + "                  \n"
                + "                </li>\n"
                + "              \n"
                + "            </ul>\n"
                + "          </div>\n"
                + "          <div id=\"user_visits\" class=\"clearfix\">\n"
                + "            <h5>最近访客 <span style='font-weight:normal;font-size:12px;padding-left:30px;'><a href=\"/blog/user_visits\">更多访客&gt;&gt;</a></span></h5>\n"
                + "            \n"
                + "              <div class=\"user_visit\">\n"
                + "                <div class=\"logo\"><a href='http://flashsword20.iteye.com' target='_blank'><img alt=\"flashsword20的博客\" class=\"logo\" src=\"http://www.iteye.com/images/user-logo-thumb.gif?1324994303\" title=\"flashsword20的博客: \" /></a></div>\n"
                + "                <div class=\"left\"><a href='http://flashsword20.iteye.com' target='_blank' title='flashsword20'>flashsword20</a></div>\n"
                + "              </div>\n"
                + "            \n"
                + "              <div class=\"user_visit\">\n"
                + "                <div class=\"logo\"><a href='http://dylinshi126.iteye.com' target='_blank'><img alt=\"dylinshi126的博客\" class=\"logo\" src=\"http://www.iteye.com/images/user-logo-thumb.gif?1324994303\" title=\"dylinshi126的博客: \" /></a></div>\n"
                + "                <div class=\"left\"><a href='http://dylinshi126.iteye.com' target='_blank' title='dylinshi126'>dylinshi126</a></div>\n"
                + "              </div>\n"
                + "            \n"
                + "              <div class=\"user_visit\">\n"
                + "                <div class=\"logo\"><a href='http://machoo.iteye.com' target='_blank'><img alt=\"machoo的博客\" class=\"logo\" src=\"http://www.iteye.com/upload/logo/user/590501/f3e5a6de-fa04-3ca9-92bd-378230b128c8-thumb.jpg?1321544632\" title=\"machoo的博客: 虚拟机终结者\" /></a></div>\n"
                + "                <div class=\"left\"><a href='http://machoo.iteye.com' target='_blank' title='machoo'>machoo</a></div>\n"
                + "              </div>\n"
                + "            \n"
                + "              <div class=\"user_visit\">\n"
                + "                <div class=\"logo\"><a href='http://arson.iteye.com' target='_blank'><img alt=\"arson的博客\" class=\"logo\" src=\"http://www.iteye.com/upload/logo/user/511499/91eafa67-ebbb-32d2-a1c4-fc1c169b5c66-thumb.jpg?1310020715\" title=\"arson的博客: \" /></a></div>\n"
                + "                <div class=\"left\"><a href='http://arson.iteye.com' target='_blank' title='arson'>arson</a></div>\n"
                + "              </div>\n"
                + "            \n"
                + "          </div>\n"
                + "\n"
                + "          \n"
                + "\n"
                + "                      <div id=\"blog_menu\">\n"
                + "              <h5>文章分类</h5>\n"
                + "              <ul>\n"
                + "                <li><a href=\"/\">全部博客 (10)</a></li>\n"
                + "                \n"
                + "                  <li><a href=\"/category/180178\">java (1)</a></li>\n"
                + "                \n"
                + "                  <li><a href=\"/category/178810\">JavaScript (2)</a></li>\n"
                + "                \n"
                + "                  <li><a href=\"/category/181978\">SQLServer (1)</a></li>\n"
                + "                \n"
                + "                  <li><a href=\"/category/214133\">MySQL (1)</a></li>\n"
                + "                \n"
                + "                  <li><a href=\"/category/182324\">爬虫 (1)</a></li>\n"
                + "                \n"
                + "                  <li><a href=\"/category/195652\">ibatis (2)</a></li>\n"
                + "                \n"
                + "                  <li><a href=\"/category/195881\">Spring (1)</a></li>\n"
                + "                \n"
                + "                  <li><a href=\"/category/216639\">tomcat (0)</a></li>\n"
                + "                \n"
                + "                  <li><a href=\"/category/217595\">数据结构 (1)</a></li>\n"
                + "                \n"
                + "              </ul>\n"
                + "            </div>\n"
                + "            <div id='month_blogs'>\n"
                + "              <h5>社区版块</h5>\n"
                + "              <ul>\n"
                + "                <li><a href=\"/blog/news\">我的资讯</a> (0)</li>\n"
                + "                <li>\n"
                + "                  <a href=\"/blog/post\">我的论坛</a> (0)\n"
                + "                </li>\n"
                + "                <li><a href=\"/blog/answered_problems\">我的问答</a> (0)</li>\n"
                + "              </ul>\n"
                + "            </div>\n"
                + "            <div id=\"month_blogs\">\n"
                + "              <h5>存档分类</h5>\n"
                + "              <ul>\n"
                + "                \n"
                + "                  <li><a href=\"/blog/monthblog/2012-04\">2012-04</a> (2)</li>\n"
                + "                \n"
                + "                  <li><a href=\"/blog/monthblog/2012-03\">2012-03</a> (1)</li>\n"
                + "                \n"
                + "                  <li><a href=\"/blog/monthblog/2012-02\">2012-02</a> (1)</li>\n"
                + "                \n"
                + "                <li><a href=\"/blog/monthblog_more\">更多存档...</a></li>\n"
                + "              </ul>\n"
                + "            </div>\n"
                + "            \n"
                + "            \n"
                + "\n"
                + "            <div id=\"guest_books\">\n"
                + "              <h5>最新评论</h5>\n"
                + "              <ul>\n"
                + "                \n"
                + "                <li>\n"
                + "                  <a href='http://marrymyy.iteye.com' target='_blank' title='marrymyy'>marrymyy</a>： \n"
                + "                  太好了，刚遇到这个问题，有用<br />\n"
                + "                  <a href=\"/blog/1189699#bc2305339\">尚未备份数据库 &quot;***&quot; 的日志尾部。如果该日志包含您不希望丢失的工作，请使用 BACKUP LOG WITH NORECOVERY 备份该日志。请使用 RE</a>\n"
                + "                </li>\n"
                + "                \n"
                + "              </ul>\n"
                + "            </div>\n"
                + "\n"
                + "            <div class=\"local_bottom\"></div>\n"
                + "          \n"
                + "        </div>\n"
                + "      </div>\n"
                + "\n"
                + "      <div id=\"footer\" class=\"clearfix\">\n"
                + "        <div id=\"copyright\">\n"
                + "          <hr/>\n"
                + "          声明：ITeye文章版权属于作者，受法律保护。没有作者书面许可不得转载。若作者同意转载，必须以超链接形式标明文章原始出处和作者。<br />\n"
                + "          &copy; 2003-2012 ITeye.com.   All rights reserved.  [ 京ICP证110151号  京公网安备110105010620 ]\n"
                + "        </div>\n"
                + "      </div>\n"
                + "    </div>\n"
                + "    <script type=\"text/javascript\">\n"
                + "  document.write(\"<img src='http://stat.iteye.com/?url=\"+ encodeURIComponent(document.location.href) + \"&referrer=\" + encodeURIComponent(document.referrer) + \"&user_id=635408' width='0' height='0' />\");\n"
                + "</script>\n" + "\n" + "    \n" + "    \n" + "  </body>\n" + "</html>\n";
        String text2 = "<div>aaa</div>";
        XpathSelector xpathSelector = new XpathSelector(
                "//div[@id='main']/div[@class='blog_main']/div[@class='blog_title']/h3/a/text()");
        String select = xpathSelector.select(text);
        Assert.assertEquals("jsoup 解析页面商品信息", select);
    }

    @Test
    public void testOschina() {
        Html html1 = new Html(html);
        Assert.assertEquals("再次吐槽easyui", html1.xpath("//*[@class='QTitle']/h1/a/text()").toString());
        Assert.assertNotNull(html1.$("a[href]").xpath("//@href").all());
        Selectors.xpath("/abc/").select("");
    }

    @Test
    public void testXPath2() {
        String text = "<h1>眉山：扎实推进农业农村工作 促农持续增收<br>\n" +
                "<span>2013-07-31 23:29:45&nbsp;&nbsp;&nbsp;来源：<a href=\"http://www.mshw.net\" target=\"_blank\" style=\"color:#AAA\">眉山网</a>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;责任编辑：张斯炜</span></h1>";
        Xpath2Selector xpathSelector = new Xpath2Selector("//h1/text()");
        Assert.assertEquals("眉山：扎实推进农业农村工作 促农持续增收", xpathSelector.select(text));
    }

    @Test
    public void testXpath2Selector() {
        Xpath2Selector xpath2Selector = new Xpath2Selector("//xhtml:a/@href");
        String select = xpath2Selector.select(html);
        Assert.assertEquals("http://www.oschina.net/", select);

        List<String> selectList = xpath2Selector.selectList(html);
        Assert.assertEquals(113, selectList.size());
        Assert.assertEquals("http://www.oschina.net/", selectList.get(0));
    }

    @Ignore("take long time")
    @Test
    public void performanceTest() {
        Xpath2Selector xpath2Selector = new Xpath2Selector("//a");
        long time = System.currentTimeMillis();
        for (int i = 0; i < 1000; i++) {
            xpath2Selector.selectList(html);
        }
        System.out.println(System.currentTimeMillis() - time);

        XpathSelector xpathSelector = new XpathSelector("//a");
        time = System.currentTimeMillis();
        for (int i = 0; i < 1000; i++) {
            xpathSelector.selectList(html);
        }
        System.out.println(System.currentTimeMillis() - time);

        time = System.currentTimeMillis();
        for (int i = 0; i < 1000; i++) {
            xpath2Selector.selectList(html);
        }
        System.out.println(System.currentTimeMillis() - time);

        CssSelector cssSelector = new CssSelector("a");
        time = System.currentTimeMillis();
        for (int i = 0; i < 1000; i++) {
            cssSelector.selectList(html);
        }
        System.out.println("css " + (System.currentTimeMillis() - time));
    }

    @Ignore("take long time")
    @Test
    public void parserPerformanceTest() throws XPatherException {
        System.out.println(html.length());

        HtmlCleaner htmlCleaner = new HtmlCleaner();
        TagNode tagNode = htmlCleaner.clean(html);
        Document document = Jsoup.parse(html);

        long time = System.currentTimeMillis();
        for (int i = 0; i < 2000; i++) {
            htmlCleaner.clean(html);
        }
        System.out.println(System.currentTimeMillis() - time);

        time = System.currentTimeMillis();
        for (int i = 0; i < 2000; i++) {
            tagNode.evaluateXPath("//a");
        }
        System.out.println(System.currentTimeMillis() - time);

        System.out.println("=============");

        time = System.currentTimeMillis();
        for (int i = 0; i < 2000; i++) {
            Jsoup.parse(html);
        }
        System.out.println(System.currentTimeMillis() - time);

        time = System.currentTimeMillis();
        for (int i = 0; i < 2000; i++) {
            document.select("a");
        }
        System.out.println(System.currentTimeMillis() - time);

        System.out.println("=============");

        time = System.currentTimeMillis();
        for (int i = 0; i < 2000; i++) {
            htmlCleaner.clean(html);
        }
        System.out.println(System.currentTimeMillis() - time);

        time = System.currentTimeMillis();
        for (int i = 0; i < 2000; i++) {
            tagNode.evaluateXPath("//a");
        }
        System.out.println(System.currentTimeMillis() - time);

        System.out.println("=============");

        XPathEvaluator compile = Xsoup.compile("//a");
        time = System.currentTimeMillis();
        for (int i = 0; i < 2000; i++) {
            compile.evaluate(document);
        }
        System.out.println(System.currentTimeMillis() - time);

    }

    /**
     * New api test
     *
     * @author hooy
     * @since 8.0
     */
    private String rank = "<div class=\"container\"><div class=\"container-bd\"><div class=\"c-left\"><div class=\"periods\"><a class=\"active\" href=\"http://www.ruoxia.com/top/dianji/day\">日</a> <a href=\"http://www.ruoxia.com/top/dianji/week\">周</a> <a href=\"http://www.ruoxia.com/top/dianji/month\">月</a></div><h1 class=\"page-title\"><i class=\"icon icon-rank\"></i> <span class=\"rankTitle\">点击榜</span></h1><div data-collect-id=\"2550\" class=\"mod mod-clean pattern-update-list update-list\"><div class=\"bd\"><table><thead><tr><th width=\"30\">排名</th><th width=\"50\">分类</th><th>书名/最新章节</th><th width=\"60\">作者</th><th width=\"80\">推荐</th><th width=\"100\">更新时间</th></tr></thead><tbody><tr><td class=\"index\">1.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=54\" class=\"tag\" targe=\"_blank\">现实</a></td><td><div class=\"range\"><a data-collect-index=\"1\" target=\"_blank\" href=\"http://www.ruoxia.com/book/83981\" targe=\"_blank\" class=\"name\">校园妖孽高手</a> <a href=\"http://www.ruoxia.com/book/83981/2154682\" class=\"chapter\">第三十章 求你收我为徒</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/17562559\" targe=\"_blank\" class=\"author\">白色风帆</a></div></td><td><div>0</div></td><td><span class=\"time\">11-24 22:32</span></td></tr><tr class=\"even\"><td class=\"index\">2.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=15\" class=\"tag\" targe=\"_blank\">架空</a></td><td><div class=\"range\"><a data-collect-index=\"2\" target=\"_blank\" href=\"http://www.ruoxia.com/book/43462\" targe=\"_blank\" class=\"name\">凤谋图</a> <a href=\"http://www.ruoxia.com/book/43462/1141799\" class=\"chapter\">写给最亲爱的你们（完结感言+新书推荐）</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/4482112\" targe=\"_blank\" class=\"author\">斑陆离</a></div></td><td><div>1047</div></td><td><span class=\"time\">03-04 14:44</span></td></tr><tr><td class=\"index\">3.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=54\" class=\"tag\" targe=\"_blank\">现实</a></td><td><div class=\"range\"><a data-collect-index=\"3\" target=\"_blank\" href=\"http://www.ruoxia.com/book/90878\" targe=\"_blank\" class=\"name\">女神的全职高手</a> <a href=\"http://www.ruoxia.com/book/90878/2491923\" class=\"chapter\">第五十一章 大结局。</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/18515896\" targe=\"_blank\" class=\"author\">白玉书生</a></div></td><td><div>0</div></td><td><span class=\"time\">07-20 09:06</span></td></tr><tr class=\"even\"><td class=\"index\">4.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=12\" class=\"tag\" targe=\"_blank\">豪门</a></td><td><div class=\"range\"><a data-collect-index=\"4\" target=\"_blank\" href=\"http://www.ruoxia.com/book/80166\" targe=\"_blank\" class=\"name\">闪婚夺爱：总裁老公太霸道</a> <a href=\"http://www.ruoxia.com/book/80166/2167580\" class=\"chapter\">第140章 大结局</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/5152286\" targe=\"_blank\" class=\"author\">丛慕然</a></div></td><td><div>0</div></td><td><span class=\"time\">12-03 09:12</span></td></tr><tr><td class=\"index\">5.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=54\" class=\"tag\" targe=\"_blank\">现实</a></td><td><div class=\"range\"><a data-collect-index=\"5\" target=\"_blank\" href=\"http://www.ruoxia.com/book/84765\" targe=\"_blank\" class=\"name\">最强神眼</a> <a href=\"http://www.ruoxia.com/book/84765/2261859\" class=\"chapter\">第72章 没羞没臊（大结局）</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/3822610\" targe=\"_blank\" class=\"author\">枫长弦</a></div></td><td><div>0</div></td><td><span class=\"time\">02-01 21:12</span></td></tr><tr class=\"even\"><td class=\"index\">6.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=24\" class=\"tag\" targe=\"_blank\">玄奇</a></td><td><div class=\"range\"><a data-collect-index=\"6\" target=\"_blank\" href=\"http://www.ruoxia.com/book/41149\" targe=\"_blank\" class=\"name\">胎楼</a> <a href=\"http://www.ruoxia.com/book/41149/1135302\" class=\"chapter\">写在最后的私话</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/5095127\" targe=\"_blank\" class=\"author\">丫丫雅雅</a></div></td><td><div>3455</div></td><td><span class=\"time\">02-28 12:31</span></td></tr><tr><td class=\"index\">7.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=24\" class=\"tag\" targe=\"_blank\">玄奇</a></td><td><div class=\"range\"><a data-collect-index=\"7\" target=\"_blank\" href=\"http://www.ruoxia.com/book/40863\" targe=\"_blank\" class=\"name\">我的未婚夫白狐大人</a> <a href=\"http://www.ruoxia.com/book/40863/1180864\" class=\"chapter\">【免费公告】新书《道士房东，快开门》已经发布了</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/5058618\" targe=\"_blank\" class=\"author\">佚之狐</a></div></td><td><div>20614</div></td><td><span class=\"time\">03-31 12:37</span></td></tr><tr class=\"even\"><td class=\"index\">8.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=50\" class=\"tag\" targe=\"_blank\">复仇</a></td><td><div class=\"range\"><a data-collect-index=\"8\" target=\"_blank\" href=\"http://www.ruoxia.com/book/30816\" targe=\"_blank\" class=\"name\">艳骨</a> <a href=\"http://www.ruoxia.com/book/30816/769427\" class=\"chapter\">番外（司浔）</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/1729575\" targe=\"_blank\" class=\"author\">e小调</a></div></td><td><div>55</div></td><td><span class=\"time\">06-03 11:43</span></td></tr><tr><td class=\"index\">9.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=17\" class=\"tag\" targe=\"_blank\">穿越</a></td><td><div class=\"range\"><a data-collect-index=\"9\" target=\"_blank\" href=\"http://www.ruoxia.com/book/82131\" targe=\"_blank\" class=\"name\">王爷，别动粗</a> <a href=\"http://www.ruoxia.com/book/82131/2097402\" class=\"chapter\">第041章 看来这个女人是在乎他的</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/15972439\" targe=\"_blank\" class=\"author\">春亦盎然</a></div></td><td><div>0</div></td><td><span class=\"time\">10-27 18:50</span></td></tr><tr class=\"even\"><td class=\"index\">10.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=19\" class=\"tag\" targe=\"_blank\">宫斗</a></td><td><div class=\"range\"><a data-collect-index=\"10\" target=\"_blank\" href=\"http://www.ruoxia.com/book/52279\" targe=\"_blank\" class=\"name\">深宫胭脂乱</a> <a href=\"http://www.ruoxia.com/book/52279/1567850\" class=\"chapter\">【263】为夫来的，可还算及时 HE版，感谢小伙伴们大半年来的不离不弃！</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/1664188\" targe=\"_blank\" class=\"author\">糖小贩</a></div></td><td><div>320</div></td><td><span class=\"time\">10-31 13:58</span></td></tr><tr><td class=\"index\">11.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=19\" class=\"tag\" targe=\"_blank\">宫斗</a></td><td><div class=\"range\"><a data-collect-index=\"11\" target=\"_blank\" href=\"http://www.ruoxia.com/book/45621\" targe=\"_blank\" class=\"name\">深宫安容传</a> <a href=\"http://www.ruoxia.com/book/45621/1361138\" class=\"chapter\">番外4 韶华不负，生生世世 （有红包哟）</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/5146663\" targe=\"_blank\" class=\"author\">鱼墨</a></div></td><td><div>6268</div></td><td><span class=\"time\">07-12 20:23</span></td></tr><tr class=\"even\"><td class=\"index\">12.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=54\" class=\"tag\" targe=\"_blank\">现实</a></td><td><div class=\"range\"><a data-collect-index=\"12\" target=\"_blank\" href=\"http://www.ruoxia.com/book/69000\" targe=\"_blank\" class=\"name\">我们的爱，未完待续</a> <a href=\"http://www.ruoxia.com/book/69000/1680336\" class=\"chapter\">第175章 番外：人生没有太晚的开始</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/5152286\" targe=\"_blank\" class=\"author\">丛慕然</a></div></td><td><div>0</div></td><td><span class=\"time\">01-18 23:00</span></td></tr><tr><td class=\"index\">13.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=35\" class=\"tag\" targe=\"_blank\">婚恋</a></td><td><div class=\"range\"><a data-collect-index=\"13\" target=\"_blank\" href=\"http://www.ruoxia.com/book/65082\" targe=\"_blank\" class=\"name\">我在时光深处忘记你</a> <a href=\"http://www.ruoxia.com/book/65082/1631623\" class=\"chapter\">新书已开</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/5222423\" targe=\"_blank\" class=\"author\">柯三岁</a></div></td><td><div>0</div></td><td><span class=\"time\">12-14 20:50</span></td></tr><tr class=\"even\"><td class=\"index\">14.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=22\" class=\"tag\" targe=\"_blank\">修真</a></td><td><div class=\"range\"><a data-collect-index=\"14\" target=\"_blank\" href=\"http://www.ruoxia.com/book/85911\" targe=\"_blank\" class=\"name\">女神总裁的妖孽兵王</a> <a href=\"http://www.ruoxia.com/book/85911/2265301\" class=\"chapter\">第52章 好吃的？</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/5507678\" targe=\"_blank\" class=\"author\">包仙人</a></div></td><td><div>0</div></td><td><span class=\"time\">02-03 23:40</span></td></tr><tr><td class=\"index\">15.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=12\" class=\"tag\" targe=\"_blank\">豪门</a></td><td><div class=\"range\"><a data-collect-index=\"15\" target=\"_blank\" href=\"http://www.ruoxia.com/book/83315\" targe=\"_blank\" class=\"name\">攻心为上：薄情总裁求放过</a> <a href=\"http://www.ruoxia.com/book/83315/2126598\" class=\"chapter\">第21章：别让我讨厌你</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/17393569\" targe=\"_blank\" class=\"author\">七月晚笙</a></div></td><td><div>0</div></td><td><span class=\"time\">11-06 23:38</span></td></tr><tr class=\"even\"><td class=\"index\">16.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=17\" class=\"tag\" targe=\"_blank\">穿越</a></td><td><div class=\"range\"><a data-collect-index=\"16\" target=\"_blank\" href=\"http://www.ruoxia.com/book/39361\" targe=\"_blank\" class=\"name\">妙手萌妃</a> <a href=\"http://www.ruoxia.com/book/39361/1021373\" class=\"chapter\">九十六</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/4048268\" targe=\"_blank\" class=\"author\">糖酱不甩</a></div></td><td><div>191</div></td><td><span class=\"time\">12-02 23:37</span></td></tr><tr><td class=\"index\">17.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=17\" class=\"tag\" targe=\"_blank\">穿越</a></td><td><div class=\"range\"><a data-collect-index=\"17\" target=\"_blank\" href=\"http://www.ruoxia.com/book/35111\" targe=\"_blank\" class=\"name\">炮灰女配二嫁攻略</a> <a href=\"http://www.ruoxia.com/book/35111/942024\" class=\"chapter\">番外</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/4165342\" targe=\"_blank\" class=\"author\">烬相思</a></div></td><td><div>412</div></td><td><span class=\"time\">10-13 22:39</span></td></tr><tr class=\"even\"><td class=\"index\">18.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=12\" class=\"tag\" targe=\"_blank\">豪门</a></td><td><div class=\"range\"><a data-collect-index=\"18\" target=\"_blank\" href=\"http://www.ruoxia.com/book/46464\" targe=\"_blank\" class=\"name\">娇妻别逃</a> <a href=\"http://www.ruoxia.com/book/46464/1344044\" class=\"chapter\">系列文开更了！</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/5625273\" targe=\"_blank\" class=\"author\">九竹</a></div></td><td><div>635</div></td><td><span class=\"time\">07-01 13:15</span></td></tr><tr><td class=\"index\">19.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=15\" class=\"tag\" targe=\"_blank\">架空</a></td><td><div class=\"range\"><a data-collect-index=\"19\" target=\"_blank\" href=\"http://www.ruoxia.com/book/26318\" targe=\"_blank\" class=\"name\">妾惊华</a> <a href=\"http://www.ruoxia.com/book/26318/789360\" class=\"chapter\">番外 3 正式完结 鬼月篇</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/2970776\" targe=\"_blank\" class=\"author\">温柔的小白兔</a></div></td><td><div>144</div></td><td><span class=\"time\">06-18 09:35</span></td></tr><tr class=\"even\"><td class=\"index\">20.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=20\" class=\"tag\" targe=\"_blank\">宅斗</a></td><td><div class=\"range\"><a data-collect-index=\"20\" target=\"_blank\" href=\"http://www.ruoxia.com/book/46115\" targe=\"_blank\" class=\"name\">废妾青瑶</a> <a href=\"http://www.ruoxia.com/book/46115/1420123\" class=\"chapter\">冷玉•世上堪哀只有痴 （不无聊依旧发着玩）</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/3428612\" targe=\"_blank\" class=\"author\">梦中说梦</a></div></td><td><div>1032</div></td><td><span class=\"time\">08-15 19:03</span></td></tr><tr><td class=\"index\">21.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=19\" class=\"tag\" targe=\"_blank\">宫斗</a></td><td><div class=\"range\"><a data-collect-index=\"21\" target=\"_blank\" href=\"http://www.ruoxia.com/book/78428\" targe=\"_blank\" class=\"name\">帝女长乐</a> <a href=\"http://www.ruoxia.com/book/78428/2057572\" class=\"chapter\">第一百六十三章大结局</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/5672126\" targe=\"_blank\" class=\"author\">梅花香雨</a></div></td><td><div>0</div></td><td><span class=\"time\">09-30 20:32</span></td></tr><tr class=\"even\"><td class=\"index\">22.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=12\" class=\"tag\" targe=\"_blank\">豪门</a></td><td><div class=\"range\"><a data-collect-index=\"22\" target=\"_blank\" href=\"http://www.ruoxia.com/book/72124\" targe=\"_blank\" class=\"name\">总裁在上，萌妻不乖</a> <a href=\"http://www.ruoxia.com/book/72124/1880164\" class=\"chapter\">新文公告</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/5152286\" targe=\"_blank\" class=\"author\">丛慕然</a></div></td><td><div>0</div></td><td><span class=\"time\">06-05 11:31</span></td></tr><tr><td class=\"index\">23.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=47\" class=\"tag\" targe=\"_blank\">重生</a></td><td><div class=\"range\"><a data-collect-index=\"23\" target=\"_blank\" href=\"http://www.ruoxia.com/book/63141\" targe=\"_blank\" class=\"name\">重生之只想好好爱你</a> <a href=\"http://www.ruoxia.com/book/63141/1604556\" class=\"chapter\">第三百三十一章 明白心意</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/2149297\" targe=\"_blank\" class=\"author\">忘忧草</a></div></td><td><div>80</div></td><td><span class=\"time\">11-25 19:56</span></td></tr><tr class=\"even\"><td class=\"index\">24.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=49\" class=\"tag\" targe=\"_blank\">异世</a></td><td><div class=\"range\"><a data-collect-index=\"24\" target=\"_blank\" href=\"http://www.ruoxia.com/book/45001\" targe=\"_blank\" class=\"name\">末世之活下去</a> <a href=\"http://www.ruoxia.com/book/45001/1073956\" class=\"chapter\">第一百一十四章　大结局之另种结局</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/4876856\" targe=\"_blank\" class=\"author\">清风随意</a></div></td><td><div>68</div></td><td><span class=\"time\">01-12 10:06</span></td></tr><tr><td class=\"index\">25.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=12\" class=\"tag\" targe=\"_blank\">豪门</a></td><td><div class=\"range\"><a data-collect-index=\"25\" target=\"_blank\" href=\"http://www.ruoxia.com/book/70099\" targe=\"_blank\" class=\"name\">以婚试爱：总裁老公太过分</a> <a href=\"http://www.ruoxia.com/book/70099/1871138\" class=\"chapter\">第二百章 温哥华的暖冬</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/14023478\" targe=\"_blank\" class=\"author\">乔慕燃</a></div></td><td><div>0</div></td><td><span class=\"time\">05-29 18:46</span></td></tr><tr class=\"even\"><td class=\"index\">26.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=35\" class=\"tag\" targe=\"_blank\">婚恋</a></td><td><div class=\"range\"><a data-collect-index=\"26\" target=\"_blank\" href=\"http://www.ruoxia.com/book/51499\" targe=\"_blank\" class=\"name\">顾瑾，我们要好好的</a> <a href=\"http://www.ruoxia.com/book/51499/2122738\" class=\"chapter\">新书~婚不谈爱，总裁老公住隔壁</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/2099877\" targe=\"_blank\" class=\"author\">一朵66</a></div></td><td><div>2778</div></td><td><span class=\"time\">11-04 17:48</span></td></tr><tr><td class=\"index\">27.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=1\" class=\"tag\" targe=\"_blank\">玄奇</a></td><td><div class=\"range\"><a data-collect-index=\"27\" target=\"_blank\" href=\"http://www.ruoxia.com/book/62293\" targe=\"_blank\" class=\"name\">吟尸调妃</a> <a href=\"http://www.ruoxia.com/book/62293/1620092\" class=\"chapter\">作品最后</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/6052301\" targe=\"_blank\" class=\"author\">乔沫若轩</a></div></td><td><div>207</div></td><td><span class=\"time\">12-06 16:57</span></td></tr><tr class=\"even\"><td class=\"index\">28.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=17\" class=\"tag\" targe=\"_blank\">穿越</a></td><td><div class=\"range\"><a data-collect-index=\"28\" target=\"_blank\" href=\"http://www.ruoxia.com/book/32090\" targe=\"_blank\" class=\"name\">嫡女皇商</a> <a href=\"http://www.ruoxia.com/book/32090/1065024\" class=\"chapter\">皇商开始修文</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/2595590\" targe=\"_blank\" class=\"author\">十七帝</a></div></td><td><div>260</div></td><td><span class=\"time\">01-04 23:26</span></td></tr><tr><td class=\"index\">29.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=12\" class=\"tag\" targe=\"_blank\">豪门</a></td><td><div class=\"range\"><a data-collect-index=\"29\" target=\"_blank\" href=\"http://www.ruoxia.com/book/80386\" targe=\"_blank\" class=\"name\">豪门养女：总裁请息怒</a> <a href=\"http://www.ruoxia.com/book/80386/2174667\" class=\"chapter\">第一百三十五章 最终番外</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/2079910\" targe=\"_blank\" class=\"author\">杨家小呆</a></div></td><td><div>0</div></td><td><span class=\"time\">12-07 21:39</span></td></tr><tr class=\"even\"><td class=\"index\">30.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=15\" class=\"tag\" targe=\"_blank\">架空</a></td><td><div class=\"range\"><a data-collect-index=\"30\" target=\"_blank\" href=\"http://www.ruoxia.com/book/46774\" targe=\"_blank\" class=\"name\">这盛世，如你所愿</a> <a href=\"http://www.ruoxia.com/book/46774/1282874\" class=\"chapter\">【免费公告】完结感言+新书公告</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/4950247\" targe=\"_blank\" class=\"author\">南风知意</a></div></td><td><div>1127</div></td><td><span class=\"time\">06-06 17:28</span></td></tr><tr><td class=\"index\">31.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=17\" class=\"tag\" targe=\"_blank\">穿越</a></td><td><div class=\"range\"><a data-collect-index=\"31\" target=\"_blank\" href=\"http://www.ruoxia.com/book/34887\" targe=\"_blank\" class=\"name\">江湖有晴天</a> <a href=\"http://www.ruoxia.com/book/34887/901292\" class=\"chapter\">第一二零章 阴谋叠加，尘埃落定（结局篇）</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/2077970\" targe=\"_blank\" class=\"author\">洛紫晴</a></div></td><td><div>113</div></td><td><span class=\"time\">09-13 09:06</span></td></tr><tr class=\"even\"><td class=\"index\">32.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=15\" class=\"tag\" targe=\"_blank\">架空</a></td><td><div class=\"range\"><a data-collect-index=\"32\" target=\"_blank\" href=\"http://www.ruoxia.com/book/57202\" targe=\"_blank\" class=\"name\">江山不若美人顾</a> <a href=\"http://www.ruoxia.com/book/57202/1711384\" class=\"chapter\">番外：最终章</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/2067719\" targe=\"_blank\" class=\"author\">慕容以泽</a></div></td><td><div>597</div></td><td><span class=\"time\">02-14 18:47</span></td></tr><tr><td class=\"index\">33.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=24\" class=\"tag\" targe=\"_blank\">玄奇</a></td><td><div class=\"range\"><a data-collect-index=\"33\" target=\"_blank\" href=\"http://www.ruoxia.com/book/54323\" targe=\"_blank\" class=\"name\">一碰冥婚</a> <a href=\"http://www.ruoxia.com/book/54323/1280258\" class=\"chapter\">第一百三十九章 秦无极诡上身</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/4782768\" targe=\"_blank\" class=\"author\">二手玫瑰</a></div></td><td><div>528</div></td><td><span class=\"time\">06-04 22:04</span></td></tr><tr class=\"even\"><td class=\"index\">34.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=17\" class=\"tag\" targe=\"_blank\">穿越</a></td><td><div class=\"range\"><a data-collect-index=\"34\" target=\"_blank\" href=\"http://www.ruoxia.com/book/27977\" targe=\"_blank\" class=\"name\">侯门丑媳</a> <a href=\"http://www.ruoxia.com/book/27977/774342\" class=\"chapter\">第202章 大结局</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/666311\" targe=\"_blank\" class=\"author\">东方怡然</a></div></td><td><div>328</div></td><td><span class=\"time\">06-06 22:09</span></td></tr><tr><td class=\"index\">35.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=15\" class=\"tag\" targe=\"_blank\">架空</a></td><td><div class=\"range\"><a data-collect-index=\"35\" target=\"_blank\" href=\"http://www.ruoxia.com/book/26287\" targe=\"_blank\" class=\"name\">将本红妆</a> <a href=\"http://www.ruoxia.com/book/26287/756383\" class=\"chapter\">完结感言 我们新文见</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/2382839\" targe=\"_blank\" class=\"author\">故城阿九</a></div></td><td><div>539</div></td><td><span class=\"time\">05-24 14:42</span></td></tr><tr class=\"even\"><td class=\"index\">36.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=15\" class=\"tag\" targe=\"_blank\">架空</a></td><td><div class=\"range\"><a data-collect-index=\"36\" target=\"_blank\" href=\"http://www.ruoxia.com/book/79251\" targe=\"_blank\" class=\"name\">妃卿非故：世子，有事好商量</a> <a href=\"http://www.ruoxia.com/book/79251/2304572\" class=\"chapter\">最后的一点小内容</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/5270097\" targe=\"_blank\" class=\"author\">白玉琼楼</a></div></td><td><div>0</div></td><td><span class=\"time\">03-05 23:27</span></td></tr><tr><td class=\"index\">37.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=17\" class=\"tag\" targe=\"_blank\">穿越</a></td><td><div class=\"range\"><a data-collect-index=\"37\" target=\"_blank\" href=\"http://www.ruoxia.com/book/31901\" targe=\"_blank\" class=\"name\">美人潋滟</a> <a href=\"http://www.ruoxia.com/book/31901/1998239\" class=\"chapter\">新文《爱有余温，触手可及》</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/1412126\" targe=\"_blank\" class=\"author\">冷在</a></div></td><td><div>3215</div></td><td><span class=\"time\">08-21 16:38</span></td></tr><tr class=\"even\"><td class=\"index\">38.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=19\" class=\"tag\" targe=\"_blank\">宫斗</a></td><td><div class=\"range\"><a data-collect-index=\"38\" target=\"_blank\" href=\"http://www.ruoxia.com/book/52159\" targe=\"_blank\" class=\"name\">后宫长梧传</a> <a href=\"http://www.ruoxia.com/book/52159/1402619\" class=\"chapter\">160 大结局</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/6660633\" targe=\"_blank\" class=\"author\">嘟嘟爱吃鱼</a></div></td><td><div>905</div></td><td><span class=\"time\">08-04 20:24</span></td></tr><tr><td class=\"index\">39.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=24\" class=\"tag\" targe=\"_blank\">玄奇</a></td><td><div class=\"range\"><a data-collect-index=\"39\" target=\"_blank\" href=\"http://www.ruoxia.com/book/51408\" targe=\"_blank\" class=\"name\">冥夫别过来</a> <a href=\"http://www.ruoxia.com/book/51408/1380765\" class=\"chapter\">新文速递（我好慌）</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/2538685\" targe=\"_blank\" class=\"author\">陌妖</a></div></td><td><div>1328</div></td><td><span class=\"time\">07-25 10:58</span></td></tr><tr class=\"even\"><td class=\"index\">40.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=17\" class=\"tag\" targe=\"_blank\">穿越</a></td><td><div class=\"range\"><a data-collect-index=\"40\" target=\"_blank\" href=\"http://www.ruoxia.com/book/44908\" targe=\"_blank\" class=\"name\">公子好倾城</a> <a href=\"http://www.ruoxia.com/book/44908/1094556\" class=\"chapter\">番外</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/5193966\" targe=\"_blank\" class=\"author\">颜箴言</a></div></td><td><div>203</div></td><td><span class=\"time\">01-27 20:53</span></td></tr><tr><td class=\"index\">41.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=19\" class=\"tag\" targe=\"_blank\">宫斗</a></td><td><div class=\"range\"><a data-collect-index=\"41\" target=\"_blank\" href=\"http://www.ruoxia.com/book/33791\" targe=\"_blank\" class=\"name\">女相倾天下</a> <a href=\"http://www.ruoxia.com/book/33791/879406\" class=\"chapter\">第九十三章 大结局</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/3970105\" targe=\"_blank\" class=\"author\">睡梦之城</a></div></td><td><div>407</div></td><td><span class=\"time\">08-31 09:03</span></td></tr><tr class=\"even\"><td class=\"index\">42.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=20\" class=\"tag\" targe=\"_blank\">宅斗</a></td><td><div class=\"range\"><a data-collect-index=\"42\" target=\"_blank\" href=\"http://www.ruoxia.com/book/28208\" targe=\"_blank\" class=\"name\">庶辜</a> <a href=\"http://www.ruoxia.com/book/28208/727884\" class=\"chapter\">请假通知</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/1434153\" targe=\"_blank\" class=\"author\">君醉</a></div></td><td><div>16</div></td><td><span class=\"time\">05-03 17:38</span></td></tr><tr><td class=\"index\">43.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=12\" class=\"tag\" targe=\"_blank\">豪门</a></td><td><div class=\"range\"><a data-collect-index=\"43\" target=\"_blank\" href=\"http://www.ruoxia.com/book/79201\" targe=\"_blank\" class=\"name\">隐婚蜜爱：首席老公别太坏</a> <a href=\"http://www.ruoxia.com/book/79201/2130842\" class=\"chapter\">第138章 筱雅，我终于等到你了！</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/15011710\" targe=\"_blank\" class=\"author\">古月初雪</a></div></td><td><div>0</div></td><td><span class=\"time\">11-10 08:00</span></td></tr><tr class=\"even\"><td class=\"index\">44.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=35\" class=\"tag\" targe=\"_blank\">婚恋</a></td><td><div class=\"range\"><a data-collect-index=\"44\" target=\"_blank\" href=\"http://www.ruoxia.com/book/76667\" targe=\"_blank\" class=\"name\">予你爱情，还我光阴</a> <a href=\"http://www.ruoxia.com/book/76667/1937700\" class=\"chapter\">第066章 结尾</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/15544088\" targe=\"_blank\" class=\"author\">美人折</a></div></td><td><div>0</div></td><td><span class=\"time\">07-12 21:37</span></td></tr><tr><td class=\"index\">45.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=15\" class=\"tag\" targe=\"_blank\">架空</a></td><td><div class=\"range\"><a data-collect-index=\"45\" target=\"_blank\" href=\"http://www.ruoxia.com/book/73761\" targe=\"_blank\" class=\"name\">王妃，王爷又来拆墙了</a> <a href=\"http://www.ruoxia.com/book/73761/1908547\" class=\"chapter\">有红包</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/5672126\" targe=\"_blank\" class=\"author\">梅花香雨</a></div></td><td><div>0</div></td><td><span class=\"time\">06-23 21:02</span></td></tr><tr class=\"even\"><td class=\"index\">46.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=24\" class=\"tag\" targe=\"_blank\">玄奇</a></td><td><div class=\"range\"><a data-collect-index=\"46\" target=\"_blank\" href=\"http://www.ruoxia.com/book/42459\" targe=\"_blank\" class=\"name\">爱上我的阴阳先生</a> <a href=\"http://www.ruoxia.com/book/42459/1273498\" class=\"chapter\">完本感言</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/3227798\" targe=\"_blank\" class=\"author\">魑魅魁魃</a></div></td><td><div>1382</div></td><td><span class=\"time\">05-31 20:36</span></td></tr><tr><td class=\"index\">47.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=47\" class=\"tag\" targe=\"_blank\">重生</a></td><td><div class=\"range\"><a data-collect-index=\"47\" target=\"_blank\" href=\"http://www.ruoxia.com/book/29035\" targe=\"_blank\" class=\"name\">嫡女重生</a> <a href=\"http://www.ruoxia.com/book/29035/828174\" class=\"chapter\">完结感言</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/3252807\" targe=\"_blank\" class=\"author\">小喵家的温婉</a></div></td><td><div>334</div></td><td><span class=\"time\">07-16 19:19</span></td></tr><tr class=\"even\"><td class=\"index\">48.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=35\" class=\"tag\" targe=\"_blank\">婚恋</a></td><td><div class=\"range\"><a data-collect-index=\"48\" target=\"_blank\" href=\"http://www.ruoxia.com/book/58092\" targe=\"_blank\" class=\"name\">亲爱的，我不等你了</a> <a href=\"http://www.ruoxia.com/book/58092/2105021\" class=\"chapter\">嘿，还有宝贝在吗？</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/5209133\" targe=\"_blank\" class=\"author\">依人茶</a></div></td><td><div>505</div></td><td><span class=\"time\">11-01 16:42</span></td></tr><tr><td class=\"index\">49.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=35\" class=\"tag\" targe=\"_blank\">婚恋</a></td><td><div class=\"range\"><a data-collect-index=\"49\" target=\"_blank\" href=\"http://www.ruoxia.com/book/77198\" targe=\"_blank\" class=\"name\">爱已过期：总裁前夫请放手</a> <a href=\"http://www.ruoxia.com/book/77198/2085300\" class=\"chapter\">第一百六十七章 余生有你陪伴 全文完</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/2149297\" targe=\"_blank\" class=\"author\">忘忧草</a></div></td><td><div>0</div></td><td><span class=\"time\">10-19 18:32</span></td></tr><tr class=\"even\"><td class=\"index\">50.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=12\" class=\"tag\" targe=\"_blank\">豪门</a></td><td><div class=\"range\"><a data-collect-index=\"50\" target=\"_blank\" href=\"http://www.ruoxia.com/book/56098\" targe=\"_blank\" class=\"name\">你的甜蜜，触手不及</a> <a href=\"http://www.ruoxia.com/book/56098/1476223\" class=\"chapter\">圆宝的完结感言</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/7144700\" targe=\"_blank\" class=\"author\">圆八宝</a></div></td><td><div>540</div></td><td><span class=\"time\">09-19 19:18</span></td></tr><tr><td class=\"index\">51.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=35\" class=\"tag\" targe=\"_blank\">婚恋</a></td><td><div class=\"range\"><a data-collect-index=\"51\" target=\"_blank\" href=\"http://www.ruoxia.com/book/45650\" targe=\"_blank\" class=\"name\">江太太，恋爱已生效</a> <a href=\"http://www.ruoxia.com/book/45650/1161174\" class=\"chapter\">江先生和江太太的恋城旧忆（三） 怀孕篇</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/5548454\" targe=\"_blank\" class=\"author\">呆小萌的包子</a></div></td><td><div>226</div></td><td><span class=\"time\">03-18 13:09</span></td></tr><tr class=\"even\"><td class=\"index\">52.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=17\" class=\"tag\" targe=\"_blank\">穿越</a></td><td><div class=\"range\"><a data-collect-index=\"52\" target=\"_blank\" href=\"http://www.ruoxia.com/book/28004\" targe=\"_blank\" class=\"name\">倾城不过未亡人</a> <a href=\"http://www.ruoxia.com/book/28004/1147278\" class=\"chapter\">楚木萧萧 第一章</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/2067719\" targe=\"_blank\" class=\"author\">慕容以泽</a></div></td><td><div>1026</div></td><td><span class=\"time\">03-08 16:28</span></td></tr><tr><td class=\"index\">53.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=47\" class=\"tag\" targe=\"_blank\">重生</a></td><td><div class=\"range\"><a data-collect-index=\"53\" target=\"_blank\" href=\"http://www.ruoxia.com/book/46534\" targe=\"_blank\" class=\"name\">重生女二嫁攻略</a> <a href=\"http://www.ruoxia.com/book/46534/1122895\" class=\"chapter\">算是完结感言吧</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/2595590\" targe=\"_blank\" class=\"author\">十七帝</a></div></td><td><div>304</div></td><td><span class=\"time\">02-19 10:25</span></td></tr><tr class=\"even\"><td class=\"index\">54.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=24\" class=\"tag\" targe=\"_blank\">玄奇</a></td><td><div class=\"range\"><a data-collect-index=\"54\" target=\"_blank\" href=\"http://www.ruoxia.com/book/42151\" targe=\"_blank\" class=\"name\">冥婚难测</a> <a href=\"http://www.ruoxia.com/book/42151/1117992\" class=\"chapter\">完结感言（有惊喜哦，戳进来看看）</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/1494532\" targe=\"_blank\" class=\"author\">鬼爹</a></div></td><td><div>2617</div></td><td><span class=\"time\">02-15 20:57</span></td></tr><tr><td class=\"index\">55.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=17\" class=\"tag\" targe=\"_blank\">穿越</a></td><td><div class=\"range\"><a data-collect-index=\"55\" target=\"_blank\" href=\"http://www.ruoxia.com/book/33600\" targe=\"_blank\" class=\"name\">傻王贤妃</a> <a href=\"http://www.ruoxia.com/book/33600/891372\" class=\"chapter\">感言</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/1249730\" targe=\"_blank\" class=\"author\">汐凉</a></div></td><td><div>199</div></td><td><span class=\"time\">09-04 19:43</span></td></tr><tr class=\"even\"><td class=\"index\">56.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=16\" class=\"tag\" targe=\"_blank\">同人</a></td><td><div class=\"range\"><a data-collect-index=\"56\" target=\"_blank\" href=\"http://www.ruoxia.com/book/30689\" targe=\"_blank\" class=\"name\">清宫瑾妃传</a> <a href=\"http://www.ruoxia.com/book/30689/832178\" class=\"chapter\">第220章 此生唯你（大结局）</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/1677848\" targe=\"_blank\" class=\"author\">芳小苓</a></div></td><td><div>768</div></td><td><span class=\"time\">07-19 20:00</span></td></tr><tr><td class=\"index\">57.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=20\" class=\"tag\" targe=\"_blank\">宅斗</a></td><td><div class=\"range\"><a data-collect-index=\"57\" target=\"_blank\" href=\"http://www.ruoxia.com/book/83309\" targe=\"_blank\" class=\"name\">庶女世子妃</a> <a href=\"http://www.ruoxia.com/book/83309/2273884\" class=\"chapter\">第一六五章 完美结局</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/3227798\" targe=\"_blank\" class=\"author\">魑魅魁魃</a></div></td><td><div>0</div></td><td><span class=\"time\">02-13 18:13</span></td></tr><tr class=\"even\"><td class=\"index\">58.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=12\" class=\"tag\" targe=\"_blank\">豪门</a></td><td><div class=\"range\"><a data-collect-index=\"58\" target=\"_blank\" href=\"http://www.ruoxia.com/book/80185\" targe=\"_blank\" class=\"name\">我曾卑微爱过你</a> <a href=\"http://www.ruoxia.com/book/80185/2135467\" class=\"chapter\">第96章 大结局</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/5321782\" targe=\"_blank\" class=\"author\">懒桔不懒</a></div></td><td><div>0</div></td><td><span class=\"time\">11-12 22:23</span></td></tr><tr><td class=\"index\">59.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=15\" class=\"tag\" targe=\"_blank\">架空</a></td><td><div class=\"range\"><a data-collect-index=\"59\" target=\"_blank\" href=\"http://www.ruoxia.com/book/74007\" targe=\"_blank\" class=\"name\">妾倾天下</a> <a href=\"http://www.ruoxia.com/book/74007/1962721\" class=\"chapter\">第162章 母仪天下（全书完）</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/14890712\" targe=\"_blank\" class=\"author\">璃璃</a></div></td><td><div>0</div></td><td><span class=\"time\">07-28 23:42</span></td></tr><tr class=\"even\"><td class=\"index\">60.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=35\" class=\"tag\" targe=\"_blank\">婚恋</a></td><td><div class=\"range\"><a data-collect-index=\"60\" target=\"_blank\" href=\"http://www.ruoxia.com/book/66546\" targe=\"_blank\" class=\"name\">纵使情深如故</a> <a href=\"http://www.ruoxia.com/book/66546/1697902\" class=\"chapter\">【175】一言不合就完结</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/1664188\" targe=\"_blank\" class=\"author\">糖小贩</a></div></td><td><div>0</div></td><td><span class=\"time\">02-03 23:09</span></td></tr><tr><td class=\"index\">61.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=12\" class=\"tag\" targe=\"_blank\">豪门</a></td><td><div class=\"range\"><a data-collect-index=\"61\" target=\"_blank\" href=\"http://www.ruoxia.com/book/59727\" targe=\"_blank\" class=\"name\">爱你，可念不可说</a> <a href=\"http://www.ruoxia.com/book/59727/1664270\" class=\"chapter\">完结感言</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/1494532\" targe=\"_blank\" class=\"author\">鬼爹</a></div></td><td><div>285</div></td><td><span class=\"time\">01-07 19:21</span></td></tr><tr class=\"even\"><td class=\"index\">62.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=47\" class=\"tag\" targe=\"_blank\">重生</a></td><td><div class=\"range\"><a data-collect-index=\"62\" target=\"_blank\" href=\"http://www.ruoxia.com/book/27946\" targe=\"_blank\" class=\"name\">美人皮，噬骨香</a> <a href=\"http://www.ruoxia.com/book/27946/1538607\" class=\"chapter\">【完结感言】</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/1311937\" targe=\"_blank\" class=\"author\">涣茶</a></div></td><td><div>654</div></td><td><span class=\"time\">10-12 18:16</span></td></tr><tr><td class=\"index\">63.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=43\" class=\"tag\" targe=\"_blank\">异能</a></td><td><div class=\"range\"><a data-collect-index=\"63\" target=\"_blank\" href=\"http://www.ruoxia.com/book/47521\" targe=\"_blank\" class=\"name\">末世之拐个系统做男神</a> <a href=\"http://www.ruoxia.com/book/47521/1319390\" class=\"chapter\">以后的日子 番外二</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/5776194\" targe=\"_blank\" class=\"author\">TK。优酱</a></div></td><td><div>617</div></td><td><span class=\"time\">06-18 20:23</span></td></tr><tr class=\"even\"><td class=\"index\">64.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=19\" class=\"tag\" targe=\"_blank\">宫斗</a></td><td><div class=\"range\"><a data-collect-index=\"64\" target=\"_blank\" href=\"http://www.ruoxia.com/book/50621\" targe=\"_blank\" class=\"name\">笑嫁江山</a> <a href=\"http://www.ruoxia.com/book/50621/1276556\" class=\"chapter\">第三百一十五章 伤怀</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/4819693\" targe=\"_blank\" class=\"author\">与文</a></div></td><td><div>27</div></td><td><span class=\"time\">06-02 21:05</span></td></tr><tr><td class=\"index\">65.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=28\" class=\"tag\" targe=\"_blank\">种田</a></td><td><div class=\"range\"><a data-collect-index=\"65\" target=\"_blank\" href=\"http://www.ruoxia.com/book/31900\" targe=\"_blank\" class=\"name\">重生之带着装备去种田</a> <a href=\"http://www.ruoxia.com/book/31900/886594\" class=\"chapter\">第一百三十五章 大结局</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/21456\" targe=\"_blank\" class=\"author\">灵山小道士</a></div></td><td><div>206</div></td><td><span class=\"time\">08-31 19:23</span></td></tr><tr class=\"even\"><td class=\"index\">66.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=20\" class=\"tag\" targe=\"_blank\">宅斗</a></td><td><div class=\"range\"><a data-collect-index=\"66\" target=\"_blank\" href=\"http://www.ruoxia.com/book/27944\" targe=\"_blank\" class=\"name\">恶毒女配的悠然生活</a> <a href=\"http://www.ruoxia.com/book/27944/847971\" class=\"chapter\">第240章 下辈子你还是我的妻(完)</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/3147647\" targe=\"_blank\" class=\"author\">翦语</a></div></td><td><div>2444</div></td><td><span class=\"time\">08-19 15:51</span></td></tr><tr><td class=\"index\">67.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=20\" class=\"tag\" targe=\"_blank\">宅斗</a></td><td><div class=\"range\"><a data-collect-index=\"67\" target=\"_blank\" href=\"http://www.ruoxia.com/book/28130\" targe=\"_blank\" class=\"name\">一品皇妻</a> <a href=\"http://www.ruoxia.com/book/28130/857549\" class=\"chapter\">第158章 大结局</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/3167696\" targe=\"_blank\" class=\"author\">七月白椿</a></div></td><td><div>818</div></td><td><span class=\"time\">08-07 23:38</span></td></tr><tr class=\"even\"><td class=\"index\">68.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=25\" class=\"tag\" targe=\"_blank\">现代</a></td><td><div class=\"range\"><a data-collect-index=\"68\" target=\"_blank\" href=\"http://www.ruoxia.com/book/95582\" targe=\"_blank\" class=\"name\">龙拳</a> <a href=\"http://www.ruoxia.com/book/95582/10698954\" class=\"chapter\">第一千八百七十章 大结局！ （六千字大章）</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/19990474\" targe=\"_blank\" class=\"author\"></a></div></td><td><div>0</div></td><td><span class=\"time\">12-23 17:02</span></td></tr><tr><td class=\"index\">69.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=1\" class=\"tag\" targe=\"_blank\">玄奇</a></td><td><div class=\"range\"><a data-collect-index=\"69\" target=\"_blank\" href=\"http://www.ruoxia.com/book/95598\" targe=\"_blank\" class=\"name\">民调局异闻录之勉传</a> <a href=\"http://www.ruoxia.com/book/95598/10384052\" class=\"chapter\">第四百一十七章 归宿</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/19990495\" targe=\"_blank\" class=\"author\"></a></div></td><td><div>0</div></td><td><span class=\"time\">07-23 12:00</span></td></tr><tr class=\"even\"><td class=\"index\">70.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=35\" class=\"tag\" targe=\"_blank\">婚恋</a></td><td><div class=\"range\"><a data-collect-index=\"70\" target=\"_blank\" href=\"http://www.ruoxia.com/book/69929\" targe=\"_blank\" class=\"name\">原来爱你那么伤</a> <a href=\"http://www.ruoxia.com/book/69929/2105024\" class=\"chapter\">嘿，还有宝贝在吗？</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/5209133\" targe=\"_blank\" class=\"author\">依人茶</a></div></td><td><div>0</div></td><td><span class=\"time\">11-01 16:43</span></td></tr><tr><td class=\"index\">71.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=12\" class=\"tag\" targe=\"_blank\">豪门</a></td><td><div class=\"range\"><a data-collect-index=\"71\" target=\"_blank\" href=\"http://www.ruoxia.com/book/78853\" targe=\"_blank\" class=\"name\">情深如许：霸道总裁我不约</a> <a href=\"http://www.ruoxia.com/book/78853/2029552\" class=\"chapter\">谢谢大家，新书求收藏</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/16083833\" targe=\"_blank\" class=\"author\">余无晴</a></div></td><td><div>0</div></td><td><span class=\"time\">09-12 00:01</span></td></tr><tr class=\"even\"><td class=\"index\">72.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=15\" class=\"tag\" targe=\"_blank\">架空</a></td><td><div class=\"range\"><a data-collect-index=\"72\" target=\"_blank\" href=\"http://www.ruoxia.com/book/74481\" targe=\"_blank\" class=\"name\">戾妃倾城</a> <a href=\"http://www.ruoxia.com/book/74481/1823456\" class=\"chapter\">番外1 几经辗转，故人坟草三尺高</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/14543004\" targe=\"_blank\" class=\"author\">凌家女孩</a></div></td><td><div>0</div></td><td><span class=\"time\">04-27 22:42</span></td></tr><tr><td class=\"index\">73.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=12\" class=\"tag\" targe=\"_blank\">豪门</a></td><td><div class=\"range\"><a data-collect-index=\"73\" target=\"_blank\" href=\"http://www.ruoxia.com/book/71766\" targe=\"_blank\" class=\"name\">婚久情深：闪婚娇妻深深爱</a> <a href=\"http://www.ruoxia.com/book/71766/1809712\" class=\"chapter\">第二十二章 大结局（完）</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/2079910\" targe=\"_blank\" class=\"author\">杨家小呆</a></div></td><td><div>0</div></td><td><span class=\"time\">04-19 13:55</span></td></tr><tr class=\"even\"><td class=\"index\">74.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=43\" class=\"tag\" targe=\"_blank\">异能</a></td><td><div class=\"range\"><a data-collect-index=\"74\" target=\"_blank\" href=\"http://www.ruoxia.com/book/47956\" targe=\"_blank\" class=\"name\">末世重生之女配归来</a> <a href=\"http://www.ruoxia.com/book/47956/1391378\" class=\"chapter\">第98章</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/5862910\" targe=\"_blank\" class=\"author\">慕晓玥</a></div></td><td><div>62</div></td><td><span class=\"time\">07-30 00:00</span></td></tr><tr><td class=\"index\">75.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=17\" class=\"tag\" targe=\"_blank\">穿越</a></td><td><div class=\"range\"><a data-collect-index=\"75\" target=\"_blank\" href=\"http://www.ruoxia.com/book/49277\" targe=\"_blank\" class=\"name\">皇帝你被征用了</a> <a href=\"http://www.ruoxia.com/book/49277/1373550\" class=\"chapter\">新书准备ing</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/6052301\" targe=\"_blank\" class=\"author\">乔沫若轩</a></div></td><td><div>1307</div></td><td><span class=\"time\">07-20 16:41</span></td></tr><tr class=\"even\"><td class=\"index\">76.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=24\" class=\"tag\" targe=\"_blank\">玄奇</a></td><td><div class=\"range\"><a data-collect-index=\"76\" target=\"_blank\" href=\"http://www.ruoxia.com/book/52370\" targe=\"_blank\" class=\"name\">道士房东，快开门</a> <a href=\"http://www.ruoxia.com/book/52370/1366506\" class=\"chapter\">第六百七十章 姓什么</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/5058618\" targe=\"_blank\" class=\"author\">佚之狐</a></div></td><td><div>12820</div></td><td><span class=\"time\">07-15 23:46</span></td></tr><tr><td class=\"index\">77.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=15\" class=\"tag\" targe=\"_blank\">架空</a></td><td><div class=\"range\"><a data-collect-index=\"77\" target=\"_blank\" href=\"http://www.ruoxia.com/book/43341\" targe=\"_blank\" class=\"name\">欠你一世长安</a> <a href=\"http://www.ruoxia.com/book/43341/1282888\" class=\"chapter\">【免费公告】新书《如果爱情看得见》求支持</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/4950247\" targe=\"_blank\" class=\"author\">南风知意</a></div></td><td><div>828</div></td><td><span class=\"time\">06-06 17:54</span></td></tr><tr class=\"even\"><td class=\"index\">78.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=20\" class=\"tag\" targe=\"_blank\">宅斗</a></td><td><div class=\"range\"><a data-collect-index=\"78\" target=\"_blank\" href=\"http://www.ruoxia.com/book/45958\" targe=\"_blank\" class=\"name\">鬓云香腮雪</a> <a href=\"http://www.ruoxia.com/book/45958/1256794\" class=\"chapter\">番外 倾心。（秦相宜&amp;太子）</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/5575980\" targe=\"_blank\" class=\"author\">萌七妹</a></div></td><td><div>985</div></td><td><span class=\"time\">05-20 23:53</span></td></tr><tr><td class=\"index\">79.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=24\" class=\"tag\" targe=\"_blank\">玄奇</a></td><td><div class=\"range\"><a data-collect-index=\"79\" target=\"_blank\" href=\"http://www.ruoxia.com/book/39183\" targe=\"_blank\" class=\"name\">一纸冥婚</a> <a href=\"http://www.ruoxia.com/book/39183/1198169\" class=\"chapter\">关于番外+新书</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/4782768\" targe=\"_blank\" class=\"author\">二手玫瑰</a></div></td><td><div>4960</div></td><td><span class=\"time\">04-12 15:58</span></td></tr><tr class=\"even\"><td class=\"index\">80.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=24\" class=\"tag\" targe=\"_blank\">玄奇</a></td><td><div class=\"range\"><a data-collect-index=\"80\" target=\"_blank\" href=\"http://www.ruoxia.com/book/46118\" targe=\"_blank\" class=\"name\">阴夫驾到</a> <a href=\"http://www.ruoxia.com/book/46118/1140031\" class=\"chapter\">第一六零话 最后的决战（结局篇）</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/2077970\" targe=\"_blank\" class=\"author\">洛紫晴</a></div></td><td><div>245</div></td><td><span class=\"time\">03-02 23:11</span></td></tr><tr><td class=\"index\">81.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=20\" class=\"tag\" targe=\"_blank\">宅斗</a></td><td><div class=\"range\"><a data-collect-index=\"81\" target=\"_blank\" href=\"http://www.ruoxia.com/book/39812\" targe=\"_blank\" class=\"name\">美人鬓</a> <a href=\"http://www.ruoxia.com/book/39812/1046332\" class=\"chapter\">完本公告</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/4819693\" targe=\"_blank\" class=\"author\">与文</a></div></td><td><div>34</div></td><td><span class=\"time\">12-21 10:11</span></td></tr><tr class=\"even\"><td class=\"index\">82.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=20\" class=\"tag\" targe=\"_blank\">宅斗</a></td><td><div class=\"range\"><a data-collect-index=\"82\" target=\"_blank\" href=\"http://www.ruoxia.com/book/28007\" targe=\"_blank\" class=\"name\">妻居一品</a> <a href=\"http://www.ruoxia.com/book/28007/834007\" class=\"chapter\">第259章 墩仔和淼淼（四）【大结局】</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/3154400\" targe=\"_blank\" class=\"author\">沙提子</a></div></td><td><div>1411</div></td><td><span class=\"time\">07-21 00:00</span></td></tr><tr><td class=\"index\">83.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=25\" class=\"tag\" targe=\"_blank\">现代</a></td><td><div class=\"range\"><a data-collect-index=\"83\" target=\"_blank\" href=\"http://www.ruoxia.com/book/95617\" targe=\"_blank\" class=\"name\">女总裁的超级高手</a> <a href=\"http://www.ruoxia.com/book/95617/10401594\" class=\"chapter\">第3609章、这样挺好！（大结局下）</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/19990521\" targe=\"_blank\" class=\"author\"></a></div></td><td><div>0</div></td><td><span class=\"time\">07-31 10:10</span></td></tr><tr class=\"even\"><td class=\"index\">84.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=1\" class=\"tag\" targe=\"_blank\">玄奇</a></td><td><div class=\"range\"><a data-collect-index=\"84\" target=\"_blank\" href=\"http://www.ruoxia.com/book/89974\" targe=\"_blank\" class=\"name\">活见诡</a> <a href=\"http://www.ruoxia.com/book/89974/2452074\" class=\"chapter\">第三十六章 威风凌凌</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/19016569\" targe=\"_blank\" class=\"author\">一骑妃子笑</a></div></td><td><div>0</div></td><td><span class=\"time\">06-18 13:53</span></td></tr><tr><td class=\"index\">85.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=15\" class=\"tag\" targe=\"_blank\">架空</a></td><td><div class=\"range\"><a data-collect-index=\"85\" target=\"_blank\" href=\"http://www.ruoxia.com/book/78680\" targe=\"_blank\" class=\"name\">压寨夫君请上轿</a> <a href=\"http://www.ruoxia.com/book/78680/2168844\" class=\"chapter\">103 我说了，我已经嫁过人了 大结局</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/5724868\" targe=\"_blank\" class=\"author\">唯水眠心</a></div></td><td><div>0</div></td><td><span class=\"time\">12-03 23:41</span></td></tr><tr class=\"even\"><td class=\"index\">86.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=1\" class=\"tag\" targe=\"_blank\">玄奇</a></td><td><div class=\"range\"><a data-collect-index=\"86\" target=\"_blank\" href=\"http://www.ruoxia.com/book/83856\" targe=\"_blank\" class=\"name\">九命</a> <a href=\"http://www.ruoxia.com/book/83856/2160919\" class=\"chapter\">第045章 柳家的秘密</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/2547718\" targe=\"_blank\" class=\"author\">渴雨</a></div></td><td><div>0</div></td><td><span class=\"time\">11-28 22:13</span></td></tr><tr><td class=\"index\">87.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=12\" class=\"tag\" targe=\"_blank\">豪门</a></td><td><div class=\"range\"><a data-collect-index=\"87\" target=\"_blank\" href=\"http://www.ruoxia.com/book/79904\" targe=\"_blank\" class=\"name\">暖婚契约：腹黑总裁的呆萌妻</a> <a href=\"http://www.ruoxia.com/book/79904/2128055\" class=\"chapter\">【118】大结局</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/15919521\" targe=\"_blank\" class=\"author\">落小妹</a></div></td><td><div>0</div></td><td><span class=\"time\">11-07 22:48</span></td></tr><tr class=\"even\"><td class=\"index\">88.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=35\" class=\"tag\" targe=\"_blank\">婚恋</a></td><td><div class=\"range\"><a data-collect-index=\"88\" target=\"_blank\" href=\"http://www.ruoxia.com/book/77693\" targe=\"_blank\" class=\"name\">眷你如火如荼</a> <a href=\"http://www.ruoxia.com/book/77693/2010568\" class=\"chapter\">【终章】、经年后，道一句好久不见</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/6095530\" targe=\"_blank\" class=\"author\">粟越</a></div></td><td><div>0</div></td><td><span class=\"time\">08-29 23:15</span></td></tr><tr><td class=\"index\">89.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=28\" class=\"tag\" targe=\"_blank\">种田</a></td><td><div class=\"range\"><a data-collect-index=\"89\" target=\"_blank\" href=\"http://www.ruoxia.com/book/45352\" targe=\"_blank\" class=\"name\">女儿香满田</a> <a href=\"http://www.ruoxia.com/book/45352/1998240\" class=\"chapter\">新文《爱有余温，触手可及》</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/1412126\" targe=\"_blank\" class=\"author\">冷在</a></div></td><td><div>1831</div></td><td><span class=\"time\">08-21 16:38</span></td></tr><tr class=\"even\"><td class=\"index\">90.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=12\" class=\"tag\" targe=\"_blank\">豪门</a></td><td><div class=\"range\"><a data-collect-index=\"90\" target=\"_blank\" href=\"http://www.ruoxia.com/book/74771\" targe=\"_blank\" class=\"name\">千万婚约：求你放了我！</a> <a href=\"http://www.ruoxia.com/book/74771/1936113\" class=\"chapter\">【112】</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/12820436\" targe=\"_blank\" class=\"author\">纳兰一梦</a></div></td><td><div>0</div></td><td><span class=\"time\">07-11 21:25</span></td></tr><tr><td class=\"index\">91.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=12\" class=\"tag\" targe=\"_blank\">豪门</a></td><td><div class=\"range\"><a data-collect-index=\"91\" target=\"_blank\" href=\"http://www.ruoxia.com/book/73591\" targe=\"_blank\" class=\"name\">虐爱成殇：冷傲总裁太绝情</a> <a href=\"http://www.ruoxia.com/book/73591/1892707\" class=\"chapter\">第138章 小玥，我爱你，你愿意嫁给我吗？</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/15011710\" targe=\"_blank\" class=\"author\">古月初雪</a></div></td><td><div>0</div></td><td><span class=\"time\">06-13 15:37</span></td></tr><tr class=\"even\"><td class=\"index\">92.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=12\" class=\"tag\" targe=\"_blank\">豪门</a></td><td><div class=\"range\"><a data-collect-index=\"92\" target=\"_blank\" href=\"http://www.ruoxia.com/book/71367\" targe=\"_blank\" class=\"name\">总裁的替身小萌妻</a> <a href=\"http://www.ruoxia.com/book/71367/1838552\" class=\"chapter\">178·尾声</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/366099\" targe=\"_blank\" class=\"author\">沈乔</a></div></td><td><div>0</div></td><td><span class=\"time\">05-07 22:10</span></td></tr><tr><td class=\"index\">93.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=12\" class=\"tag\" targe=\"_blank\">豪门</a></td><td><div class=\"range\"><a data-collect-index=\"93\" target=\"_blank\" href=\"http://www.ruoxia.com/book/71221\" targe=\"_blank\" class=\"name\">早安，小甜妻</a> <a href=\"http://www.ruoxia.com/book/71221/1733121\" class=\"chapter\">第71章：大结局（完）</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/14303346\" targe=\"_blank\" class=\"author\">星辉熠熠</a></div></td><td><div>0</div></td><td><span class=\"time\">02-28 00:01</span></td></tr><tr class=\"even\"><td class=\"index\">94.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=12\" class=\"tag\" targe=\"_blank\">豪门</a></td><td><div class=\"range\"><a data-collect-index=\"94\" target=\"_blank\" href=\"http://www.ruoxia.com/book/60151\" targe=\"_blank\" class=\"name\">娇妻休逃</a> <a href=\"http://www.ruoxia.com/book/60151/1632790\" class=\"chapter\">第326章 我带你飞（大结局）</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/5625273\" targe=\"_blank\" class=\"author\">九竹</a></div></td><td><div>304</div></td><td><span class=\"time\">12-16 07:30</span></td></tr><tr><td class=\"index\">95.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=35\" class=\"tag\" targe=\"_blank\">婚恋</a></td><td><div class=\"range\"><a data-collect-index=\"95\" target=\"_blank\" href=\"http://www.ruoxia.com/book/54317\" targe=\"_blank\" class=\"name\">宋先生，我的爱情已过期</a> <a href=\"http://www.ruoxia.com/book/54317/1578696\" class=\"chapter\">完结感言+新书速递《给我一场盛宴，纪念你离开》</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/868367\" targe=\"_blank\" class=\"author\">流年mengo</a></div></td><td><div>669</div></td><td><span class=\"time\">11-07 18:16</span></td></tr><tr class=\"even\"><td class=\"index\">96.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=7\" class=\"tag\" targe=\"_blank\">仙侠</a></td><td><div class=\"range\"><a data-collect-index=\"96\" target=\"_blank\" href=\"http://www.ruoxia.com/book/61509\" targe=\"_blank\" class=\"name\">神君快到碗里来</a> <a href=\"http://www.ruoxia.com/book/61509/1512646\" class=\"chapter\">110.最美是回忆</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/8528296\" targe=\"_blank\" class=\"author\">柒月绯然</a></div></td><td><div>54</div></td><td><span class=\"time\">09-25 19:51</span></td></tr><tr><td class=\"index\">97.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=12\" class=\"tag\" targe=\"_blank\">豪门</a></td><td><div class=\"range\"><a data-collect-index=\"97\" target=\"_blank\" href=\"http://www.ruoxia.com/book/58932\" targe=\"_blank\" class=\"name\">我爱你，与你无关</a> <a href=\"http://www.ruoxia.com/book/58932/1444784\" class=\"chapter\">大结局</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/10048716\" targe=\"_blank\" class=\"author\">南有木木</a></div></td><td><div>655</div></td><td><span class=\"time\">08-31 13:02</span></td></tr><tr class=\"even\"><td class=\"index\">98.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=54\" class=\"tag\" targe=\"_blank\">现实</a></td><td><div class=\"range\"><a data-collect-index=\"98\" target=\"_blank\" href=\"http://www.ruoxia.com/book/55800\" targe=\"_blank\" class=\"name\">你与时光皆薄凉</a> <a href=\"http://www.ruoxia.com/book/55800/1340844\" class=\"chapter\">完结感言</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/6517325\" targe=\"_blank\" class=\"author\">七寸南季</a></div></td><td><div>374</div></td><td><span class=\"time\">06-29 09:55</span></td></tr><tr><td class=\"index\">99.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=17\" class=\"tag\" targe=\"_blank\">穿越</a></td><td><div class=\"range\"><a data-collect-index=\"99\" target=\"_blank\" href=\"http://www.ruoxia.com/book/48522\" targe=\"_blank\" class=\"name\">夫君，你别跑</a> <a href=\"http://www.ruoxia.com/book/48522/1320612\" class=\"chapter\">完结了，完结了</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/5209133\" targe=\"_blank\" class=\"author\">依人茶</a></div></td><td><div>373</div></td><td><span class=\"time\">06-19 18:07</span></td></tr><tr class=\"even\"><td class=\"index\">100.</td><td><a target=\"_blank\" href=\"http://www.ruoxia.com/all?sort=35\" class=\"tag\" targe=\"_blank\">婚恋</a></td><td><div class=\"range\"><a data-collect-index=\"100\" target=\"_blank\" href=\"http://www.ruoxia.com/book/52762\" targe=\"_blank\" class=\"name\">彼年豆蔻，白首不离</a> <a href=\"http://www.ruoxia.com/book/52762/1280140\" class=\"chapter\">第一百四十二章大结局</a></div></td><td><div class=\"range\"><a target=\"_blank\" href=\"http://www.ruoxia.com/people/5672126\" targe=\"_blank\" class=\"author\">梅花香雨</a></div></td><td><div>159</div></td><td><span class=\"time\">06-04 21:05</span></td></tr></tbody></table></div></div></div><div class=\"c-right\"><div class=\"c-right\"><div class=\"mod mod-block sidebar-menu\"><div class=\"hd\"><h5><span>排行榜</span></h5></div><div class=\"bd\"><ul><li id=\"monthly\"><a href=\"http://www.ruoxia.com/top/monthly/day?rank=13\">钻石榜</a></li><li id=\"dianji\" class=\"active\"><a href=\"http://www.ruoxia.com/top/dianji/day\">点击榜</a></li><li id=\"tuijian\"><a href=\"http://www.ruoxia.com/top/tuijian/day?rank=3\">推荐榜</a></li><li id=\"xinshu\"><a href=\"http://www.ruoxia.com/top/xinshu/day\">新书榜</a></li><li id=\"pengchang\"><a href=\"http://www.ruoxia.com/top/pengchang/day\">捧场榜</a></li><li id=\"finishding\"><a href=\"http://www.ruoxia.com/top/finishding/day\">完本订阅榜</a></li><li id=\"wbTuijian\"><a href=\"http://www.ruoxia.com/top/wbTuijian/day?rank=16\">完本推荐榜</a></li><li id=\"wbMonthly\"><a href=\"http://www.ruoxia.com/top/wbMonthly/day?rank=17\">完本钻石榜</a></li></ul></div></div></div></div></div></div>";

    @Test
    public void testStringAPI() {
        // testAPI: selectList(String) -> selectList(Node)
        List<String> items = new Xpath2Selector("//div[@class=\"bd\"]//tbody/tr").selectList(rank);
        Assert.assertSame(100, items.size());
        // testAPI: select(String) -> select(Node)
        String name = new Xpath2Selector("//td[3]/div/a[1]/text()").select(items.get(10));
        Assert.assertEquals("深宫安容传", name);
    }

    @Test
    public void testNodeAPI() {
        // testAPI: selectNodes(String) -> selectNodes(Node)
        List<Node> items = new Xpath2Selector("//div[@class=\"bd\"]//tbody/tr").selectNodes(rank);
        Assert.assertSame(100, items.size());
        // testAPI: selectNode(Node)
        Node item = new Xpath2Selector("./td[3]/div/a[1]").selectNode(items.get(10));
        String name = new Xpath2Selector("./text()").select(item);
        Assert.assertEquals("深宫安容传", name);
    }

    @Test
    public void testUtilAPI() throws TransformerException {
        Node item = Xpath2Selector.newInstance("//div[@class=\"bd\"]//tbody/tr[11]/td[3]/div/a[1]/text()").selectNode(rank);
        // testAPI: nodeToString(Node) -> nodesToStrings(List<Node>)
        String name = JaxpSelectorUtils.nodeToString(item);
        Assert.assertEquals("深宫安容传", name);
    }

}
